﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="SampleAnonymousAdminController.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Admin.Samples
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Mvc;

    using DevExpress.Data.Filtering;
    using DevExpress.Web.Mvc;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Controllers.Interfaces;
    using Fruitmonitoring.Helper.Export;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;
    using Fruitmonitoring.Models.BindingHandlers;

    #endregion

    /// <summary>
    /// The sample anonymous admin controller.
    /// </summary>
    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Admin)]
    public partial class SampleAnonymousAdminController : BaseSampleDetailController, IExportController
    {
        #region Constructors and Destructors

        #region constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="SampleAnonymousAdminController"/> class.
        /// </summary>
        public SampleAnonymousAdminController()
            : base(
                new SampleBindingHandlers(),
                GridViewHelper.SAMPLEANONYMOUSADMINGRIDVIEWID,
                GridViewHelper.GRIDVIEWCOMPONENTSPATH + "Sample/_sampleAnonymousGridViewPartial",
                CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.Sample_id),
                GridViewHelper.SAMPLEANONYMOUSADMINCONTROLLERNAME)
        {
        }
        #endregion

        #endregion

        #region Methods

        #region private methods
        /// <summary>
        /// The get fixed filter expression.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string getFixedFilterExpression()
        {
            return null;
        }
        #endregion

        #endregion

        #region Public Methods and Operators

        #region overrides
        /// <summary>
        /// The custom binding core.
        /// </summary>
        /// <param name="gridViewModel">
        /// The grid view model.
        /// </param>
        /// <param name="sortColumn">
        /// The sort column.
        /// </param>
        /// <param name="oldPageSize">
        /// The old page size.
        /// </param>
        /// <returns>
        /// The <see cref="PartialViewResult"/>.
        /// </returns>
        public override PartialViewResult CustomBindingCore(GridViewModel gridViewModel, GridViewColumnState sortColumn = null, int oldPageSize = -1)
        {
            GridViewFixedFilterExpression = getFixedFilterExpression();
            PartialViewResult partialViewResult = base.CustomBindingCore(gridViewModel, sortColumn, oldPageSize);
            return partialViewResult;
        }
        #endregion

        #region actionresults
        /// <summary>
        /// The export.
        /// </summary>
        /// <param name="exportType">
        /// The export type.
        /// </param>
        /// <param name="columnNames">
        /// The column names.
        /// </param>
        /// <param name="columnTexts">
        /// The column texts.
        /// </param>
        /// <param name="filter">
        /// The filter.
        /// </param>
        /// <param name="sortColumnName">
        /// The sort column name.
        /// </param>
        /// <param name="sortOrder">
        /// The sort order.
        /// </param>
        /// <param name="applySorting">
        /// The apply sorting.
        /// </param>
        /// <param name="applyFilter">
        /// The apply filter.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        public ActionResult Export(
            string exportType,
            string columnNames,
            string columnTexts,
            string filter,
            string sortColumnName,
            string sortOrder,
            bool applySorting,
            bool applyFilter)
        {
            if (applyFilter)
            {
                if (string.IsNullOrEmpty(filter) || filter.ToLower().Equals("null"))
                {
                    // apply fixed filter
                    filter = getFixedFilterExpression();
                }
                else
                {
                    filter += " AND " + getFixedFilterExpression();
                }
            }
            else
            {
                // apply fixed filter
                filter = getFixedFilterExpression();
            }

            // set additional parameters
            var additionalParameters = new Dictionary<string, object>() { { SampleBigWithResultsBindingHandlers.PROPERTYKEY_USEUSERCOMPANYID, false } };

            // get data
            var sampleBindingHandlers = new SampleBigWithResultsBindingHandlers(
                applySorting ? sortColumnName : null,
                applySorting ? parseSortOrder(sortOrder) : BaseBindingHandlers.DbOrderType.NONE,
                0,
                Exporter.MAXROWS,
                GridViewFilterHelper.GenerateXmlFilter(filter, EFHelper.Usp_Sample_BigWithResults.GetType()),
                null,
                additionalParameters);
            sampleBindingHandlers.SetMembershipUser(User);
            var model = sampleBindingHandlers.LazyModel;

            // export
            return Export(model, columnNames, columnTexts, (CustomMembershipUser)User);
        }
        #endregion

        #endregion

        #region overrides BaseSampleController
        /// <summary>
        /// The sample detail tab view.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailTabView(Guid id)
        {
            return SampleDetails(id, User.GetAccountGuid(), null, User.GetCompanyGuid(), SampleViewType.TABVIEW, false);
        }

        /// <summary>
        /// The sample details print view.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailsPrintView(Guid id)
        {
            return SampleDetails(id, User.GetAccountGuid(), null, User.GetCompanyGuid(), SampleViewType.PRINT, false);
        }

        /// <summary>
        /// The sample detail information.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="showAnonymous">
        /// The show anonymous.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailInformation(USP_Sample_Sel_Result model, bool showAnonymous = true)
        {
            return base.SampleDetailInformation(model, true);
        }

        /// <summary>
        /// The sample detail documents.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [ChildActionOnly]
        public override ActionResult SampleDetailDocuments(List<USP_SampleDocuments_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail certification quality seals.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailCertificationQualitySeals(List<USP_Sample2CertificationQualitySeal_Sel_Result> model)
        {
            return base.SampleDetailCertificationQualitySeals(model);
        }

        /// <summary>
        /// The sample detail certification business.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [ChildActionOnly]
        public override ActionResult SampleDetailCertificationBusiness(List<USP_Sample2CertificationBusiness_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail certification products.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [ChildActionOnly]
        public override ActionResult SampleDetailCertificationProducts(List<USP_Sample2CertificationProduct_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail history.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [ChildActionOnly]
        public override ActionResult SampleDetailHistory(List<USP_SampleHistory_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail results.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailResults(List<USP_SampleResults_Sel_Result> model)
        {
            return base.SampleDetailResults(model);
        }

        /// <summary>
        /// The sample detail ratings.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailRatings(List<USP_SampleRating_Sel_Result> model)
        {
            return base.SampleDetailRatings(model);
        }

        /// <summary>
        /// The sample detail single elements.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailSingleElements(List<USP_SingleElements_Sel_Result> model)
        {
            return base.SampleDetailSingleElements(model);
        }

        /// <summary>
        /// The sample detail commments.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="sampleId">
        /// The sample id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [NonAction]
        public override ActionResult SampleDetailCommments(List<USP_SampleComments_Sel_Result> model, Guid sampleId)
        {
            return null;
        }
        #endregion
    }
}