﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="BaseAccountManagementController.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.General
{
    #region usings
    using System;
    using System.Web.Mvc;

    using DevExpress.Data;

    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;
    using Fruitmonitoring.Models.BindingHandlers;

    using Languages.Emails.Accounts;

    #endregion

    /// <summary>
    /// The base account management controller.
    /// </summary>
    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Admin | FruitmonitoringRoles.SuperUserLaboratory)]
    public abstract class BaseAccountManagementController : BaseGridViewCustomBindingController
    {
        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="BaseAccountManagementController"/> class.
        /// </summary>
        /// <param name="bindingHandler">
        /// The binding handler.
        /// </param>
        /// <param name="gridViewId">
        /// The grid view id.
        /// </param>
        /// <param name="gridViewPartialPath">
        /// The grid view partial path.
        /// </param>
        /// <param name="keyFieldName">
        /// The key field name.
        /// </param>
        /// <param name="controllerName">
        /// The controller name.
        /// </param>
        /// <param name="modelType">
        /// The model type.
        /// </param>
        /// <param name="defaultSortColumn">
        /// The default sort column.
        /// </param>
        /// <param name="defaultSortColumnSortOrder">
        /// The default sort column sort order.
        /// </param>
        /// <param name="isAdmin">
        /// The is admin.
        /// </param>
        /// <param name="isSuperUser">
        /// The is super user.
        /// </param>
        public BaseAccountManagementController(
            IBindingHandler bindingHandler,
            string gridViewId,
            string gridViewPartialPath,
            string keyFieldName,
            string controllerName,
            Type modelType,
            string defaultSortColumn,
            ColumnSortOrder defaultSortColumnSortOrder,
            bool isAdmin,
            bool isSuperUser)
            : base(
                bindingHandler,
                gridViewId,
                gridViewPartialPath,
                keyFieldName,
                controllerName,
                modelType,
                defaultSortColumn,
                defaultSortColumnSortOrder,
                isAdmin,
                isSuperUser)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BaseAccountManagementController"/> class.
        /// </summary>
        /// <param name="bindingHandler">
        /// The binding handler.
        /// </param>
        /// <param name="gridViewId">
        /// The grid view id.
        /// </param>
        /// <param name="gridViewPartialPath">
        /// The grid view partial path.
        /// </param>
        /// <param name="keyFieldName">
        /// The key field name.
        /// </param>
        /// <param name="controllerName">
        /// The controller name.
        /// </param>
        /// <param name="modelType">
        /// The model type.
        /// </param>
        /// <param name="isAdmin">
        /// The is admin.
        /// </param>
        /// <param name="isSuperUser">
        /// The is super user.
        /// </param>
        public BaseAccountManagementController(
            IBindingHandler bindingHandler,
            string gridViewId,
            string gridViewPartialPath,
            string keyFieldName,
            string controllerName,
            Type modelType,
            bool isAdmin,
            bool isSuperUser)
            : base(bindingHandler, gridViewId, gridViewPartialPath, keyFieldName, controllerName, modelType, isAdmin, isSuperUser)
        {
        }
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// Send an mail with the generated password to the new added user.
        /// </summary>
        /// <param name="model">
        /// New account model.
        /// </param>
        /// <param name="generatedPassword">
        /// The generated password.
        /// </param>
        public void SendPasswordMail(USP_Account_Sel_Result model, string generatedPassword)
        {
        }
        #endregion
    }
}