﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="BaseDashboardController.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.General
{
    #region usings
    using System.Linq;
    using System.Web.Mvc;

    using DevExpress.Data;
    using DevExpress.Data.Filtering;
    using DevExpress.Data.Linq;
    using DevExpress.Data.Linq.Helpers;

    using Fruitmonitoring.Helper.HtmlExtensions.PeriodOfTimeSelection;
    using Fruitmonitoring.Infrastructure.ActionAttributes;

    #endregion

    /// <summary>
    /// The base dashboard controller.
    /// </summary>
    public abstract class BaseDashboardController : BaseGridViewController, IPeriodOfTimeSelection
    {
        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="BaseDashboardController"/> class.
        /// constructor provides important informations to the view.
        /// </summary>
        /// <param name="controllerName">
        /// Name of the controller, which should handle the incomming gridview requests.
        /// </param>
        /// <param name="gridViewId">
        /// The unique gridview retialerCompanyId.
        /// </param>
        public BaseDashboardController(string controllerName, string gridViewId)
            : base(controllerName, gridViewId, false, false)
        {
        }
        #endregion

        #region Methods

        #region methods
        /// <summary>
        /// Sort a IQueryable.
        /// </summary>
        /// <param name="model">
        /// The IQeuerable.
        /// </param>
        /// <param name="columnName">
        /// Property name that shoeld be sorted.
        /// </param>
        /// <typeparam name="T">
        /// Type of elements in IQueryable.
        /// </typeparam>
        /// <returns>
        /// Returns a sorted IQeueryable.
        /// </returns>
        protected IQueryable<T> SetDefaultSorting<T>(IQueryable<T> model, string columnName) where T : class
        {
            return null;
        }
        #endregion

        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// The grid view list view.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        public abstract override ActionResult GridViewListView();

        /// <summary>
        /// The period list view.
        /// </summary>
        /// <param name="selectionValue">
        /// The selection value.
        /// </param>
        /// <param name="period">
        /// The period.
        /// </param>
        /// <param name="withCurrentYear">
        /// The with current year.
        /// </param>
        /// <param name="filterExpression">
        /// The gridviews filter expression.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public virtual ActionResult PeriodListView(string selectionValue = "", int period = -1, bool withCurrentYear = false, string filterExpression = "")
        {
            return null;
        }
        #endregion
    }
}