﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="BaseSampleController.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.General
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Data.Entity.Core.Objects;
    using System.IO;
    using System.Linq;
    using System.Text;
    using System.Web;
    using System.Web.Mvc;

    using DevExpress.Data;
    using DevExpress.Data.Filtering;
    using DevExpress.Web.Mvc;

    using Fruitmonitoring.Helper.Export;
    using Fruitmonitoring.Helper.FileHelper;
    using Fruitmonitoring.Helper.Modules.SampleTransferObservers;
    using Fruitmonitoring.Helper.Sample;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.AccessRules;
    using Fruitmonitoring.Infrastructure.AccessRules.Interface;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;
    using Fruitmonitoring.Models.BindingHandlers;
    using Fruitmonitoring.Models.ComboBox;
    using Fruitmonitoring.Models.Validation;

    using Languages;
    using Languages.Pages.AllAreas.InvestigationOrder;
    using Languages.Pages.AllAreas.Samples;

    #endregion

    /// <summary>
    /// The base sample controller.
    /// </summary>
    public abstract partial class BaseSampleController : BaseGridViewCustomBindingController
    {
        #region variables
        /// <summary>
        /// The validationsingleelements.
        /// </summary>
        public static readonly string VALIDATIONSINGLEELEMENTS = "tb" + GridViewHelper.PRODUCTSINGLEELEMENTSMANAGEMENTGRIDVIEWID;

        /// <summary>
        /// The validationsampleratings.
        /// </summary>
        public static readonly string VALIDATIONSAMPLERATINGS = "tb" + GridViewHelper.SAMPLERATINGMANAGEMENTGRIDVIEWID;

        /// <summary>
        /// The validationsampleresults.
        /// </summary>
        public static readonly string VALIDATIONSAMPLERESULTS = "tb" + GridViewHelper.SAMPLERESULTSMANAGEMENTGRIDVIEWID;

        /// <summary>
        /// The validationsampledocuments.
        /// </summary>
        public static readonly string VALIDATIONSAMPLEDOCUMENTS = "tb" + GridViewHelper.DOCUMENTFILEUPLOADERGRIDVIEWID;

        /// <summary>
        /// The validationinvestigationparameter.
        /// </summary>
        public static readonly string VALIDATIONINVESTIGATIONPARAMETER = "tb" + GridViewHelper.INVESTIGATIONPARAMETERGRIDVIEWID;
        #endregion

        #region constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="BaseSampleController"/> class.
        /// </summary>
        /// <param name="bindingHandler">
        /// The binding handler.
        /// </param>
        /// <param name="gridViewId">
        /// The grid view id.
        /// </param>
        /// <param name="gridViewPartialPath">
        /// The grid view partial path.
        /// </param>
        /// <param name="keyFieldName">
        /// The key field name.
        /// </param>
        /// <param name="activeControllerName">
        /// The active controller name.
        /// </param>
        public BaseSampleController(
            IBindingHandler bindingHandler,
            string gridViewId,
            string gridViewPartialPath,
            string keyFieldName,
            string activeControllerName)
            : base(bindingHandler, gridViewId, gridViewPartialPath, keyFieldName, activeControllerName, EFHelper.Usp_Sample.GetType(), false, false)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BaseSampleController"/> class.
        /// </summary>
        /// <param name="bindingHandler">
        /// The binding handler.
        /// </param>
        /// <param name="gridViewId">
        /// The grid view id.
        /// </param>
        /// <param name="gridViewPartialPath">
        /// The grid view partial path.
        /// </param>
        /// <param name="keyFieldName">
        /// The key field name.
        /// </param>
        /// <param name="activeControllerName">
        /// The active controller name.
        /// </param>
        /// <param name="defaultSortColumn">
        /// The default sort column.
        /// </param>
        /// <param name="defaultSortColumnSortOrder">
        /// The default sort column sort order.
        /// </param>
        public BaseSampleController(
            IBindingHandler bindingHandler,
            string gridViewId,
            string gridViewPartialPath,
            string keyFieldName,
            string activeControllerName,
            string defaultSortColumn,
            ColumnSortOrder defaultSortColumnSortOrder)
            : base(
                bindingHandler,
                gridViewId,
                gridViewPartialPath,
                keyFieldName,
                activeControllerName,
                EFHelper.Usp_Sample.GetType(),
                defaultSortColumn,
                defaultSortColumnSortOrder,
                false,
                false)
        {
        }
        #endregion

        #region methods
        /// <summary>
        /// Change the status of the selected investigation orders.
        /// </summary>
        /// <param name="newStatus">
        /// New status.
        /// </param>
        /// <param name="orderIds">
        /// List with order-ids.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        [NonAction]
        public bool ChangeInvestigationOrderStatus(Guid newStatus, List<Guid> orderIds)
        {
            return false;
        }

        /// <summary>
        /// Change the status of the selected samples.
        /// </summary>
        /// <param name="newStatus">
        /// New status.
        /// </param>
        /// <param name="samplesIds">
        /// List with sample-ids.
        /// </param>
        /// <param name="commentCancellation">
        /// The comment Cancellation.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        [NonAction]
        public bool ChangeSampleStatus(Guid newStatus, List<Guid> samplesIds, string commentCancellation)
        {
            return false;
        }

        /// <summary>
        /// Generates the xml for the stored procedure that is needed to change the sample status.
        /// </summary>
        /// <param name="newStatus">
        /// New sample status.
        /// </param>
        /// <param name="ids">
        /// New status will set to this samples.
        /// </param>
        /// <param name="commentCancellation">
        /// Comment for sample cancellation.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        [NonAction]
        protected string createStatusChangeXml(Guid newStatus, List<Guid> ids, string commentCancellation)
        {
            return null;
        }

        /// <summary>
        /// Change the status of the selected samples.
        /// </summary>
        /// <param name="newStatus">
        /// New status.
        /// </param>
        /// <param name="samplesIds">
        /// Sample/order ids.
        /// </param>
        /// <param name="succeedMsg">
        /// Users succeed message.
        /// </param>
        /// <param name="failedMsg">
        /// Useres error message.
        /// </param>
        /// <param name="commentCancellation">
        /// Status cancellation comment.
        /// </param>
        /// <param name="isInvestigationOrder">
        /// The is Investigation Order.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        [NonAction]
        private bool changeStatus(
            Guid newStatus,
            List<Guid> samplesIds,
            string succeedMsg,
            string failedMsg,
            string commentCancellation,
            bool isInvestigationOrder)
        {
            return false;
        }

        /// <summary>
        /// Return only the samples that have the status given with "statusId".
        /// </summary>
        /// <param name="statusIds">
        /// Status that should accepted.
        /// </param>
        /// <param name="sampleList">
        /// Sample list.
        /// </param>
        /// <returns>
        /// Sample list that have the status "statusId".
        /// </returns>
        [NonAction]
        public List<USP_Sample_Sel_Result> DeleteSamplesFromListWithWrongStatus(List<Guid> statusIds, List<USP_Sample_Sel_Result> sampleList)
        {
            return null;
        }

        /// <summary>
        /// Return only the samples that have not the status provided as parameter.
        /// </summary>
        /// <param name="statusId">
        /// The status Id.
        /// </param>
        /// <param name="sampleList">
        /// Sample list.
        /// </param>
        /// <returns>
        /// Sample list without the samples with the status provided.
        /// </returns>
        [NonAction]
        public List<USP_Sample_Sel_Result> DeleteSamplesFromListWithStatus(Guid statusId, List<USP_Sample_Sel_Result> sampleList)
        {
            return null;
        }

        /// <summary>
        /// Send mails to admin and laboratories if sample status was changed by admin.
        /// </summary>
        /// <param name="newStatus">
        /// </param>
        /// <param name="samples">
        /// </param>
        /// <param name="commentCancellation">
        /// If storno, the cancellation comment must provided.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string SendMailsChangeSampleStatus(Guid newStatus, List<USP_Sample_Sel_Result> samples, string commentCancellation)
        {
            return null;
        }

        /// <summary>
        /// Get xml filter for sample selection
        /// selects all samples with the ids in the sampleIds string.
        /// </summary>
        /// <param name="sampleIds">
        /// DbGuidHelper.JQUERYLISTSEPARATOR separated string with all sample ids.
        /// </param>
        /// <returns>
        /// Xml filter that can be used in the stored procedure.
        /// </returns>
        [NonAction]
        public string GetSamplesIdFilter(string sampleIds)
        {
            return null;
        }

        /// <summary>
        /// Excel export using OpenXml.
        /// </summary>
        /// <param name="model">
        /// Model data.
        /// </param>
        /// <param name="columnNames">
        /// Properties that should be exported.
        /// </param>
        /// <param name="columnNamesTrans">
        /// Column header names for the excel export.
        /// </param>
        /// <param name="user">
        /// The user.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        private ActionResult createExcel(
            ObjectResult<USP_Sample_Sel_BigWithResults_Result> model,
            string columnNames,
            string columnNamesTrans,
            CustomMembershipUser user)
        {
            return null;
        }
        #endregion

        #region actionresults
        /// <summary>
        /// The custom binding core.
        /// </summary>
        /// <param name="gridViewModel">
        /// The grid view model.
        /// </param>
        /// <param name="sortColumn">
        /// The sort column.
        /// </param>
        /// <param name="oldPageSize">
        /// The old page size.
        /// </param>
        /// <returns>
        /// The <see cref="PartialViewResult"/>.
        /// </returns>
        public override PartialViewResult CustomBindingCore(GridViewModel gridViewModel, GridViewColumnState sortColumn = null, int oldPageSize = -1)
        {
            return null;
        }

        /// <summary>
        /// Load viewdata for gridview view.
        /// </summary>
        private void loadViewData()
        {
        }

        /// <summary>
        /// The sample view type.
        /// </summary>
        public enum SampleViewType
        {
            /// <summary>
            /// The tabview.
            /// </summary>
            TABVIEW,

            /// <summary>
            /// The print.
            /// </summary>
            PRINT
        }

        /// <summary>
        /// GET: /Sample/PagesForTabs
        /// set dataSets for all tabviews.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <param name="accountId">
        /// Current login account id.
        /// </param>
        /// <param name="companyId">
        /// Current company id. if null, not all values are returned from db.
        /// </param>
        /// <param name="sampleRatingCompanyId">
        /// Provide an company id, if you want to show the sample ratings for the company only.
        /// </param>
        /// <param name="viewType">
        /// Render details in tabview or printview.
        /// </param>
        /// <param name="allowFileUpload">
        /// The allow File Upload.
        /// </param>
        /// <returns>
        /// Pages control which holds the tab view.
        /// </returns>
        protected ActionResult SampleDetails(
            Guid id,
            Guid accountId,
            Guid? companyId,
            Guid? sampleRatingCompanyId,
            SampleViewType viewType,
            bool allowFileUpload)
        {
            return null;
        }

        /// <summary>
        /// The get sample.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <param name="accountId">
        /// The account id.
        /// </param>
        /// <param name="companyId">
        /// The company id.
        /// </param>
        /// <returns>
        /// The <see cref="USP_Sample_Sel_Result"/>.
        /// </returns>
        private USP_Sample_Sel_Result GetSample(Guid id, Guid accountId, Guid? companyId)
        {
            return null;
        }

        /// <summary>
        /// The get sample ratings.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <param name="sampleRatingCompanyId">
        /// The sample rating company id.
        /// </param>
        /// <param name="showOnlyFm">
        /// The show only fm.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_SampleRating_Sel_Result> GetSampleRatings(Guid id, Guid? sampleRatingCompanyId, bool showOnlyFm)
        {
            return null;
        }

        /// <summary>
        /// The get sample history.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_SampleHistory_Sel_Result> GetSampleHistory(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The get sample comments.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_SampleComments_Sel_Result> GetSampleComments(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The get sample certification quality seals.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_Sample2CertificationQualitySeal_Sel_Result> GetSampleCertificationQualitySeals(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The get sample certification product.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_Sample2CertificationProduct_Sel_Result> GetSampleCertificationProduct(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The get sample certification business.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_Sample2CertificationBusiness_Sel_Result> GetSampleCertificationBusiness(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The get sample documents public.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <param name="companyId">
        /// The company id.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_SampleDocuments_Sel_Result> GetSampleDocumentsPublic(Guid id, Guid companyId)
        {
            return null;
        }

        /// <summary>
        /// The get sample documents company.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <param name="companyId">
        /// The company id.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_SampleDocuments_Sel_Result> GetSampleDocumentsCompany(Guid id, Guid companyId)
        {
            return null;
        }

        /// <summary>
        /// The get sample results.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_SampleResults_Sel_Result> GetSampleResults(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The get sample single elements.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_SingleElements_Sel_Result> GetSampleSingleElements(Guid id)
        {
            return null;
        }

        /// <summary>
        /// Get sample detail information partial view.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="showAnonymous">
        /// Show the anonymous sample information.
        /// </param>
        /// <returns>
        /// Page with content for attachments.
        /// </returns>
        public virtual ActionResult SampleDetailInformation(USP_Sample_Sel_Result model, bool showAnonymous = true)
        {
            return null;
        }

        /// <summary>
        /// Get sample documents partial view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <returns>
        /// Page with content for attachments.
        /// </returns>
        public virtual ActionResult SampleDetailDocuments(List<USP_SampleDocuments_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// Get sample documents uploader partial view.
        /// </summary>
        /// <param name="sampleId">
        /// Sample id.
        /// </param>
        /// <param name="companyId">
        /// Show documents for company.
        /// </param>
        /// <param name="showDeleteButton">
        /// Is the delete button available.
        /// </param>
        /// <returns>
        /// Page with document uploader.
        /// </returns>
        public virtual ActionResult SampleDetailDocumentsUploader(Guid sampleId, Guid companyId, bool showDeleteButton = false)
        {
            return null;
        }

        /// <summary>
        /// Get quality seal partial view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <returns>
        /// Page with content for qualityseals.
        /// </returns>
        public virtual ActionResult SampleDetailCertificationQualitySeals(List<USP_Sample2CertificationQualitySeal_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// Get certification business partial view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <returns>
        /// Page with content for CertificationBusiness.
        /// </returns>
        public virtual ActionResult SampleDetailCertificationBusiness(List<USP_Sample2CertificationBusiness_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// Get certification product partial view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <returns>
        /// Page with content for CertificationProduct.
        /// </returns>
        public virtual ActionResult SampleDetailCertificationProducts(List<USP_Sample2CertificationProduct_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// Get detail history partial view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <returns>
        /// Page with content for history.
        /// </returns>
        public virtual ActionResult SampleDetailHistory(List<USP_SampleHistory_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail history detail.
        /// </summary>
        /// <param name="historyId">
        /// The history id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [AjaxOrChildActionOnly]
        [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Admin)]
        public ActionResult SampleDetailHistoryDetail(Guid historyId)
        {
            return null;
        }

        /// <summary>
        /// Get detail result partial view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <returns>
        /// Page with content for results.
        /// </returns>
        public virtual ActionResult SampleDetailResults(List<USP_SampleResults_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// Get detail result partial view first parts of columns
        /// needed for print view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <returns>
        /// Page with content for results.
        /// </returns>
        public virtual ActionResult SampleDetailResultsSplitPart1(List<USP_SampleResults_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// Get detail result partial view second parts of columns
        /// needed for print view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <returns>
        /// Page with content for results.
        /// </returns>
        public virtual ActionResult SampleDetailResultsSplitPart2(List<USP_SampleResults_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// Get detail rating partial view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <returns>
        /// Page with content for ratings.
        /// </returns>
        public virtual ActionResult SampleDetailRatings(List<USP_SampleRating_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// Get detail single elements partial view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <returns>
        /// Page with content for ratings.
        /// </returns>
        public virtual ActionResult SampleDetailSingleElements(List<USP_SingleElements_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// Get detail notices partial view.
        /// </summary>
        /// <param name="model">
        /// Enumeration which holds the data objects.
        /// </param>
        /// <param name="sampleId">
        /// The sample Id.
        /// </param>
        /// <returns>
        /// Page with content for results.
        /// </returns>
        public virtual ActionResult SampleDetailCommments(List<USP_SampleComments_Sel_Result> model, Guid sampleId)
        {
            return null;
        }

        /// <summary>
        /// The export.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="columnNames">
        /// The column names.
        /// </param>
        /// <param name="columnNamesTrans">
        /// The column names trans.
        /// </param>
        /// <param name="user">
        /// The user.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [ChildActionOnly]
        public ActionResult Export(
            ObjectResult<USP_Sample_Sel_BigWithResults_Result> model,
            string columnNames,
            string columnNamesTrans,
            CustomMembershipUser user)
        {
            return null;
        }

        /// <summary>
        /// Get delegate sp for sample edit.
        /// </summary>
        /// <param name="model">
        /// Sample model.
        /// </param>
        /// <param name="retailerSelectionIds">
        /// Retailer ids.
        /// </param>
        /// <param name="certificationBusinessSelectionIds">
        /// Certification business ids.
        /// </param>
        /// <param name="certificationProductSelectionIds">
        /// Certification product ids.
        /// </param>
        /// <param name="certificationQualitySealSelectionIds">
        /// Certification qualilty seal ids.
        /// </param>
        /// <returns>
        /// Delegate function to edit a model.
        /// </returns>
        public DelegateEdit GetDelegateEditSample(
            USP_Sample_Sel_Result model,
            string[] retailerSelectionIds,
            string[] certificationBusinessSelectionIds,
            string[] certificationProductSelectionIds,
            string[] certificationQualitySealSelectionIds)
        {
            return null;
        }

        /// <summary>
        /// Get delegate sp for sample add.
        /// </summary>
        /// <param name="model">
        /// Sample model.
        /// </param>
        /// <param name="retailerSelectionIds">
        /// Retailer ids.
        /// </param>
        /// <param name="certificationBusinessSelectionIds">
        /// Certification business ids.
        /// </param>
        /// <param name="certificationProductSelectionIds">
        /// Certification product ids.
        /// </param>
        /// <param name="certificationQualitySealSelectionIds">
        /// Certification qualilty seal ids.
        /// </param>
        /// <returns>
        /// Delegate function to edit a model.
        /// </returns>
        public DelegateAdd GetDelegateAddSample(
            USP_Sample_Sel_Result model,
            string[] retailerSelectionIds,
            string[] certificationBusinessSelectionIds,
            string[] certificationProductSelectionIds,
            string[] certificationQualitySealSelectionIds)
        {
            return null;
        }

        /// <summary>
        /// The get delegate add investigation order.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="retailerIds">
        /// The retailer ids.
        /// </param>
        /// <param name="certificationBusinessIds">
        /// The certification business ids.
        /// </param>
        /// <param name="certificationProductIds">
        /// The certification product ids.
        /// </param>
        /// <param name="certificationQualityIds">
        /// The certification quality ids.
        /// </param>
        /// <returns>
        /// The <see cref="BaseGridViewController.DelegateAdd"/>.
        /// </returns>
        public DelegateAdd GetDelegateAddInvestigationOrder(
            USP_Sample_Investigation_Sel_Result model,
            string[] retailerIds,
            string[] certificationBusinessIds,
            string[] certificationProductIds,
            string[] certificationQualityIds)
        {
            return null;
        }
        #endregion
    }
}