﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="InvestigationOrderOpenLaboratoryController.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Laboratory.Samples
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Mvc;

    using DevExpress.Data.Filtering;
    using DevExpress.Web.Mvc;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Helper.Sample;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;
    using Fruitmonitoring.Models.BindingHandlers;

    using Languages.Pages.Laboratory.Samples.InvestigationOrder;

    using Sample = Languages.Pages.Laboratory.Samples.Samples.Sample;

    #endregion

    /// <summary>
    /// The investigation order open laboratory controller.
    /// </summary>
    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Admin | FruitmonitoringRoles.Laboratory)]
    public class InvestigationOrderOpenLaboratoryController : BaseInvestigationOrderController
    {
        #region Constructors and Destructors

        #region constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="InvestigationOrderOpenLaboratoryController"/> class.
        /// </summary>
        public InvestigationOrderOpenLaboratoryController()
            : base(
                new SampleInvestigationOrderBindingHandlers(
                    new Dictionary<string, object>() { { SampleInvestigationOrderBindingHandlers.PROPERTYKEY_USEUSERCOMPANYID, true } }),
                GridViewHelper.INVESTIGATIONORDEROPENLABORATORYGRIDVIEWID,
                GridViewHelper.GRIDVIEWCOMPONENTSPATH + "Sample/_sampleInvestigationOrderGridViewPartial",
                CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.Sample_id),
                GridViewHelper.INVESTIGATIONORDEROPENLABORATORYCONTROLLERNAME)
        {
        }
        #endregion

        #endregion

        #region Public Methods and Operators

        #region overrides
        /// <summary>
        /// The custom binding core.
        /// </summary>
        /// <param name="gridViewModel">
        /// The grid view model.
        /// </param>
        /// <param name="sortColumn">
        /// The sort column.
        /// </param>
        /// <param name="oldPageSize">
        /// The old page size.
        /// </param>
        /// <returns>
        /// The <see cref="PartialViewResult"/>.
        /// </returns>
        public override PartialViewResult CustomBindingCore(GridViewModel gridViewModel, GridViewColumnState sortColumn = null, int oldPageSize = -1)
        {
            return null;
        }
        #endregion

        #endregion

        #region Methods

        #region private methods
        /// <summary>
        /// The get fixed filter expression.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string getFixedFilterExpression()
        {
            return null;
        }
        #endregion

        #endregion

        #region action results
        /// <summary>
        /// Set the status of an investigation order to "sample receipt"
        /// info: quick and dirty :./
        /// </summary>
        /// <param name="sampleId">
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult AcceptInvestigationOrder(Guid sampleId)
        {
            return null;
        }

        /// <summary>
        /// Add a new order from a order with all single elements.
        /// </summary>
        /// <param name="sampleId">
        /// </param>
        /// <param name="order">
        /// </param>
        /// <param name="retailersIdArray">
        /// </param>
        /// <param name="certificationBusinessIdArray">
        /// </param>
        /// <param name="certificationProductIdArray">
        /// </param>
        /// <param name="certificationQualitySealsIdArray">
        /// </param>
        /// <param name="newSampleId">
        /// </param>
        /// <param name="singleElements">
        /// </param>
        private void addSampleFromOrder(
            Guid sampleId,
            USP_Sample_Investigation_Sel_Result order,
            string[] retailersIdArray,
            string[] certificationBusinessIdArray,
            string[] certificationProductIdArray,
            string[] certificationQualitySealsIdArray,
            Guid newSampleId,
            List<USP_SingleElements_Sel_Result> singleElements)
        {
        }

        /// <summary>
        /// Insert single elements for a specific sample.
        /// </summary>
        /// <param name="newSampleId">
        /// </param>
        /// <param name="singleElements">
        /// </param>
        private void CopySingleElements(Guid newSampleId, List<USP_SingleElements_Sel_Result> singleElements)
        {
        }

        /// <summary>
        /// Set the status of an investigation order to "order declined".
        /// </summary>
        /// <param name="sampleId">
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult DeclineInvestigationOrder(Guid sampleId)
        {
            return null;
        }

        /// <summary>
        /// Change status of investigation order and set info messages.
        /// </summary>
        /// <param name="investigationOrderId">
        /// </param>
        private void handleAcceptInvestigationOrderSucceed(Guid investigationOrderId)
        {
        }

        /// <summary>
        /// Get all certification quality seal ids for a specific order.
        /// </summary>
        /// <param name="sample">
        /// The sample.
        /// </param>
        /// <returns>
        /// The <see cref="string[]"/>.
        /// </returns>
        private string[] getCertificationQualitySealIds(USP_Sample_Investigation_Sel_Result sample)
        {
            var certificationQualitySeals =
                Db.USP_Sample2CertificationQualitySeal_Sel(null, sample.Sample_id, null, null, null, 0, null, null, CultureHelper.GetCultureISO639_3())
                    .ToList<USP_Sample2CertificationQualitySeal_Sel_Result>();
            var certificationQualitySealsIdArray = certificationQualitySeals.Select(c => c.CertificationQualitySeal_id.ToString()).ToArray<string>();
            return certificationQualitySealsIdArray;
        }

        /// <summary>
        /// Get all certification product ids for a specific order.
        /// </summary>
        /// <param name="sample">
        /// The sample.
        /// </param>
        /// <returns>
        /// The <see cref="string[]"/>.
        /// </returns>
        private string[] getCertificationProductIds(USP_Sample_Investigation_Sel_Result sample)
        {
            var certificationProducts =
                Db.USP_Sample2CertificationProduct_Sel(null, sample.Sample_id, null, null, null, 0, null, null, CultureHelper.GetCultureISO639_3())
                    .ToList<USP_Sample2CertificationProduct_Sel_Result>();
            var certificationProductIdArray = certificationProducts.Select(c => c.CertificationProduct_id.ToString()).ToArray<string>();
            return certificationProductIdArray;
        }

        /// <summary>
        /// Get all certification business ids for a specific order.
        /// </summary>
        /// <param name="sample">
        /// The sample.
        /// </param>
        /// <returns>
        /// The <see cref="string[]"/>.
        /// </returns>
        private string[] getCertificationBusinessIds(USP_Sample_Investigation_Sel_Result sample)
        {
            var certificationBusiness =
                Db.USP_Sample2CertificationBusiness_Sel(null, sample.Sample_id, null, null, null, 0, null, null, CultureHelper.GetCultureISO639_3())
                    .ToList<USP_Sample2CertificationBusiness_Sel_Result>();
            var certificationBusinessIdArray = certificationBusiness.Select(c => c.CertificationBusiness_id.ToString()).ToArray<string>();
            return certificationBusinessIdArray;
        }

        /// <summary>
        /// Get all retailer ids for a specific order.
        /// </summary>
        /// <param name="sample">
        /// The sample.
        /// </param>
        /// <returns>
        /// The <see cref="string[]"/>.
        /// </returns>
        private string[] getRetailerIds(USP_Sample_Investigation_Sel_Result sample)
        {
            var retailers =
                Db.USP_CompanyRetailer2Sample_Sel(null, null, sample.Sample_id, null, null, 0, null, null, CultureHelper.GetCultureISO639_3())
                    .ToList<USP_CompanyRetailer2Sample_Sel_Result>();
            var retailersIdArray = retailers.Select(r => r.Company_id.ToString()).ToArray<string>();
            return retailersIdArray;
        }
        #endregion
    }
}