﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="CompanyRetailerController.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Retailer.CustomerMasterdata
{
    #region usings
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Mvc;

    using DevExpress.Web.Mvc;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Controllers.Interfaces;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;
    using Fruitmonitoring.Models.BindingHandlers;

    #endregion

    /// <summary>
    /// The company retailer controller.
    /// </summary>
    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Retailer)]
    public class CompanyRetailerController : BaseCompanyController, IExportController
    {
        #region Constructors and Destructors

        #region constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="CompanyRetailerController"/> class.
        /// </summary>
        public CompanyRetailerController()
            : base(
                new CompanyBindingHandlers(additionalParameters),
                GridViewHelper.COMPANYRETAILERGRIDVIEWID,
                GridViewHelper.GRIDVIEWCOMPONENTSPATH + "Company/_companyGridViewPartial",
                CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Company.Company_id),
                GridViewHelper.COMPANYRETAILERCONTROLLERNAME,
                EFHelper.Usp_Company.GetType(),
                false,
                false)
        {
        }
        #endregion

        #endregion

        // define all company types that are shown in the retailer gridview here

        #region Static Fields
        /// <summary>
        /// The company types.
        /// </summary>
        private static readonly string companyTypes = string.Format(
            "{1}{0}{2}{0}{3}{0}{4}{0}{5}{0}{6}{0}{7}{0}{8}{0}{9}",
            DbGuidHelper.DBLISTSEPARATOR,
            DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.DELIVERYSTORAGE),
            DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.SENDER),
            DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.FACTORY),
            DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.LABORATORY),
            DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.PRODUCER),
            DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.RETAILERCLIENT),
            DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.RETAILERCOMPANY),
            DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.RETAILERREGION),
            DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.MANUFACTURER));

        /// <summary>
        /// The additional parameters.
        /// </summary>
        private static readonly Dictionary<string, object> additionalParameters = new Dictionary<string, object>()
                                                                                  {
                                                                                      {
                                                                                          CompanyBindingHandlers
                                                                                          .PROPERTYKEY_COMPANYTYPEIDLIST,
                                                                                          companyTypes
                                                                                      }
                                                                                  };
        #endregion

        // do not add the "SUPPLIER" type!

        #region Public Methods and Operators
        /// <summary>
        /// The custom binding core.
        /// </summary>
        /// <param name="gridViewModel">
        /// The grid view model.
        /// </param>
        /// <param name="sortColumn">
        /// The sort column.
        /// </param>
        /// <param name="oldPageSize">
        /// The old page size.
        /// </param>
        /// <returns>
        /// The <see cref="PartialViewResult"/>.
        /// </returns>
        public override PartialViewResult CustomBindingCore(GridViewModel gridViewModel, GridViewColumnState sortColumn = null, int oldPageSize = -1)
        {
            return null;
        }

        /// <summary>
        /// The export.
        /// </summary>
        /// <param name="exportType">
        /// The export type.
        /// </param>
        /// <param name="columnNames">
        /// The column names.
        /// </param>
        /// <param name="columnTexts">
        /// The column texts.
        /// </param>
        /// <param name="filter">
        /// The filter.
        /// </param>
        /// <param name="sortColumnName">
        /// The sort column name.
        /// </param>
        /// <param name="sortOrder">
        /// The sort order.
        /// </param>
        /// <param name="applySorting">
        /// The apply sorting.
        /// </param>
        /// <param name="applyFilter">
        /// The apply filter.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        public ActionResult Export(
            string exportType,
            string columnNames,
            string columnTexts,
            string filter,
            string sortColumnName,
            string sortOrder,
            bool applySorting,
            bool applyFilter)
        {
            return null;
        }

        /// <summary>
        /// Get all possible company types (retailer).
        /// </summary>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        public override List<USP_CompanyType_Sel_Result> getCompanyTypes()
        {
            return null;
        }
        #endregion
    }
}