﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="DashboardRetailerController.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Retailer
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web;
    using System.Web.Mvc;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Controllers.Interfaces;
    using Fruitmonitoring.Helper.HtmlExtensions.PeriodOfTimeSelection;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;

    #endregion

    /// <summary>
    /// The dashboard retailer controller.
    /// </summary>
    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Retailer)]
    public class DashboardRetailerController : BaseDashboardController, IExportController
    {
        #region Properties
        /// <summary>
        /// Gets or sets the period.
        /// </summary>
        private int period { get; set; }

        /// <summary>
        /// Gets or sets the selection value.
        /// </summary>
        private string selectionValue { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether with current year.
        /// </summary>
        private bool withCurrentYear { get; set; }
        #endregion

        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="DashboardRetailerController"/> class.
        /// constructor provides important informations to the view.
        /// </summary>
        /// <param name="controllerName">
        /// Name of the controller, which should handle the incomming gridview requests.
        /// </param>
        /// <param name="gridViewId">
        /// The unique gridview id.
        /// </param>
        public DashboardRetailerController()
            : base(GridViewHelper.DASHBOARDRETAILERCONTROLLERNAME, GridViewHelper.DASHBOARDPERIODRETAILERGRIDVIEWID)
        {
        }
        #endregion

        #region Fields
        private readonly string gridViewPartialFile = "Dashboard/_retailerGridViewPartial";

        private readonly string gridViewPartialFileRetailerAnonym = "Dashboard/_retailerAnonymGridViewPartial";
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// The grid view list view.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult GridViewListView()
        {
            return null;
        }

        /// <summary>
        /// The grid view list view.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult GridViewListViewInit()
        {
            return null;
        }
        #endregion

        #region IPeriodOfTimeSelection Member
        /// <summary>
        /// The period list view.
        /// </summary>
        /// <param name="selectionValue">
        /// The selection value.
        /// </param>
        /// <param name="period">
        /// The period.
        /// </param>
        /// <param name="withCurrentYear">
        /// The with current year.
        /// </param>
        /// <param name="filterExpression">
        /// The gridviews filter expression.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult PeriodListView(string selectionValue = "", int period = -1, bool withCurrentYear = false, string filterExpression = "")
        {
            return null;
        }

        /// <summary>
        /// The grid view samples anonym.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult GridViewSamplesAnonym()
        {
            return null;
        }

        /// <summary>
        /// The grid view samples anonym.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult GridViewSamplesAnonymInit()
        {
            return null;
        }
        #endregion

        #region IExportController Member
        /// <summary>
        /// The export.
        /// </summary>
        /// <param name="exportType">
        /// The export type.
        /// </param>
        /// <param name="columnNames">
        /// The column names.
        /// </param>
        /// <param name="columnTexts">
        /// The column texts.
        /// </param>
        /// <param name="filter">
        /// The filter.
        /// </param>
        /// <param name="sortColumnName">
        /// The sort column name.
        /// </param>
        /// <param name="sortOrder">
        /// The sort order.
        /// </param>
        /// <param name="applySorting">
        /// The apply sorting.
        /// </param>
        /// <param name="applyFilter">
        /// The apply filter.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        public ActionResult Export(
            string exportType,
            string columnNames,
            string columnTexts,
            string filter,
            string sortColumnName,
            string sortOrder,
            bool applySorting,
            bool applyFilter)
        {
            return null;
        }

        /// <summary>
        /// The export sample overview anonymous.
        /// </summary>
        /// <param name="exportType">
        /// The export type.
        /// </param>
        /// <param name="columnNames">
        /// The column names.
        /// </param>
        /// <param name="columnTexts">
        /// The column texts.
        /// </param>
        /// <param name="filter">
        /// The filter.
        /// </param>
        /// <param name="sortColumnName">
        /// The sort column name.
        /// </param>
        /// <param name="sortOrder">
        /// The sort order.
        /// </param>
        /// <param name="applySorting">
        /// The apply sorting.
        /// </param>
        /// <param name="applyFilter">
        /// The apply filter.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        public ActionResult ExportSampleOverviewAnonymous(
            string exportType,
            string columnNames,
            string columnTexts,
            string filter,
            string sortColumnName,
            string sortOrder,
            bool applySorting,
            bool applyFilter)
        {
            return null;
        }
        #endregion
    }
}