﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="ExportCheckboxItem.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Export
{
    #region usings
    using System;

    #endregion

    /// <summary>
    /// Represents an checkbox item for the export.
    /// </summary>
    public class ExportCheckboxItem : IComparable
    {
        #region Properties
        /// <summary>
        /// Gets or sets the items index. Can be used for sorting etc.
        /// </summary>
        public int index { get; set; }

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        public string key { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether selected.
        /// </summary>
        public bool selected { get; set; }

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        public string value { get; set; }
        #endregion

        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="ExportCheckboxItem"/> class.
        /// constructor which sets all necessary values.
        /// </summary>
        /// <param name="key">
        /// Key of the checkbox.
        /// </param>
        /// <param name="value">
        /// Value of the checkbox.
        /// </param>
        /// <param name="selected">
        /// Is checkbox selected?.
        /// </param>
        /// <param name="index">
        /// The items index.
        /// </param>
        public ExportCheckboxItem(string key, string value, bool selected, int index)
        {
            this.key = key;
            this.value = value;
            this.selected = selected;
            this.index = index;
        }
        #endregion

        #region IComparable Members

        #region Public Methods and Operators

        #region Implementation of IComparable
        /// <summary>
        /// Compares the current instance with another object of the same type and returns an integer that indicates whether the current instance precedes, follows, or occurs in the same position in the sort order as the other object.
        /// </summary>
        /// <returns>
        /// A value that indicates the relative order of the objects being compared. The return value has these meanings: Value Meaning Less than zero This instance precedes <paramref name="obj"/> in the sort order. Zero This instance occurs in the same position in the sort order as <paramref name="obj"/>. Greater than zero This instance follows <paramref name="obj"/> in the sort order. 
        /// </returns>
        /// <param name="obj">An object to compare with this instance. </param><exception cref="T:System.ArgumentException"><paramref name="obj"/> is not the same type as this instance. </exception>
        public int CompareTo(object obj)
        {
            return 0;
        }
        #endregion

        #endregion

        #endregion
    }
}