﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="DocumentFileUploaderController.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.HtmlExtensions.DocumentFileUploader
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Mvc;
    using System.Web.Routing;

    using DevExpress.Web;
    using DevExpress.Web.Mvc;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Helper.FileHelper;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;

    #endregion

    /// <summary>
    /// The document file uploader controller.
    /// </summary>
    [CustomAuthorize(
        AppRole =
            FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Admin | FruitmonitoringRoles.Laboratory | FruitmonitoringRoles.Orderer
            | FruitmonitoringRoles.Retailer)]
    public class DocumentFileUploaderController : BaseGridViewController
    {
        #region Properties

        #region Fields
        private readonly string gridViewPartialFile = "_documentFileUploaderGridViewPartial";
        #endregion

        #endregion

        #region Constructors and Destructors

        #region constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentFileUploaderController"/> class.
        /// constructor provides important informations to the view.
        /// </summary>
        /// <param name="controllerName">
        /// Name of the controller, which should handle the incomming gridview requests.
        /// </param>
        /// <param name="gridViewId">
        /// The unique gridview retialerCompanyId.
        /// </param>
        public DocumentFileUploaderController()
            : base(GridViewHelper.DOCUMENTFILEUPLOADERCONTROLLERNAME, GridViewHelper.FILEUPLOADGRIDVIEWID, false, false)
        {
        }
        #endregion

        #endregion

        #region Public Methods and Operators

        #region methods
        /// <summary>
        /// Set additional view data.
        /// </summary>
        /// <param name="sampleId">
        /// </param>
        /// <param name="showDeleteButton">
        /// The show Delete Button.
        /// </param>
        [NonAction]
        public void loadViewData(Guid? sampleId, bool showDeleteButton)
        {
        }
        #endregion

        #endregion

        #region actionresult
        /// <summary>
        /// The grid view list view.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        /// <exception cref="NotImplementedException">
        /// </exception>
        [NonAction]
        public override ActionResult GridViewListView()
        {
            return null;
        }

        /// <summary>
        /// The grid view list view.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        /// <exception cref="NotImplementedException">
        /// </exception>
        [NonAction]
        public override ActionResult GridViewListViewInit()
        {
            return null;
        }

        /// <summary>
        /// Is called before an action is executed
        /// jthis is needed to add an file-upload complete handler.
        /// </summary>
        /// <param name="requestContext">
        /// Current request context.
        /// </param>
        protected override void Initialize(RequestContext requestContext)
        {
        }

        /// <summary>
        /// Nothing to do here
        /// the fileuploadcomplete handler is added in "initialize" function.
        /// </summary>
        /// <param name="ucMultiSelection">
        /// Holds all files.
        /// </param>
        /// <param name="sampleId">
        /// Current sample id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        public ActionResult MultiSelectionUpload(IEnumerable<UploadedFile> ucMultiSelection, Guid? sampleId)
        {
            return null;
        }

        /// <summary>
        /// Show progress bar.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult MultiFileSelectionPartial()
        {
            return null;
        }

        /// <summary>
        /// Show document gridview.
        /// </summary>
        /// <param name="sampleId">
        /// Sample id.
        /// </param>
        /// <param name="showDeleteButton">
        /// Show delete button in gridview?.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult DocumentsPartial(Guid? sampleId, bool showDeleteButton)
        {
            return null;
        }

        /// <summary>
        /// Edit document description.
        /// </summary>
        /// <param name="model">
        /// </param>
        /// <param name="showDeleteButton">
        /// The show Delete Button.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult Edit(USP_SampleDocuments_Sel_Result model, bool showDeleteButton)
        {
            return null;
        }

        /// <summary>
        /// Delete a document.
        /// </summary>
        /// <param name="model">
        /// </param>
        /// <param name="sampleId">
        /// </param>
        /// <param name="showDeleteButton">
        /// The show Delete Button.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [AjaxOrChildActionOnly]
        [CustomAuthorize(AppRole = FruitmonitoringRoles.Admin | FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Laboratory | FruitmonitoringRoles.Orderer
            )]
        public ActionResult Delete(USP_SampleDocuments_Sel_Result model, Guid? sampleId, bool showDeleteButton)
        {
            return null;
        }
        #endregion
    }
}