﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="SampleHistoryParser.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Sample
{
    #region usings
    using System;
    using System.Linq;

    using Fruitmonitoring.Helper.HtmlExtensions.SamplerUserManagement;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Models;

    using log4net;

    using Languages;
    using Languages.ModelData.SampleResult;
    using Languages.ModelData.SingleElement;

    using Document = Languages.ModelData.Document.Document;
    using Sample = Languages.ModelData.Sample.Sample;
    using SampleHistory = Languages.ModelData.SampleHistory.SampleHistory;
    using SampleRating = Languages.ModelData.SampleRating.SampleRating;

    #endregion

    /// <summary>
    /// The sample history parser.
    /// </summary>
    public class SampleHistoryParser
    {
        #region Public Methods and Operators
        /// <summary>
        /// This method is used to replace the model-columnnames with readable names from languages resources
        /// moreover it replaces all id's with readable values
        /// this is really ugly code.
        /// </summary>
        /// <param name="model">
        /// The history details model that should be parsed.
        /// </param>
        /// <param name="db">
        /// Database object.
        /// </param>
        /// <returns>
        /// The <see cref="USP_SampleHistoryDetails_Sel_Result"/>.
        /// </returns>
        public USP_SampleHistoryDetails_Sel_Result ParseModelColumnNameToReadableName(USP_SampleHistoryDetails_Sel_Result model, Entities db)
        {
            var currentCulture = CultureHelper.GetCultureISO639_3();

            try
            {
                if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.LabNumber))
                {
                    model.ColumnName = Sample.LabNumber;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.FMNumber))
                {
                    model.ColumnName = Sample.FMNumber;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.OrderNumber))
                {
                    model.ColumnName = Sample.OrderNumber;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Company.Company_id)
                         && model.TableName == CustomMetadataProvider.GetMemberName(() => EFHelper.Company.CompanyRetailer2Sample))
                {
                    model.ColumnName = Sample.Retailer;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueOld), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueNew), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.CompanyLab_id))
                {
                    model.ColumnName = Sample.LaboratoryName;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueOld), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueNew), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.CompanyClient_id))
                {
                    model.ColumnName = Sample.ClientName;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueOld), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueNew), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.CompanyFactory_id))
                {
                    model.ColumnName = Sample.FactoryName;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueOld), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueNew), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.CompanyRegion_id))
                {
                    model.ColumnName = Sample.Region;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueOld), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueNew), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.Product_id))
                {
                    if (model.TableName == CustomMetadataProvider.GetMemberName(() => EFHelper.SingleElements))
                    {
                        // Distinction to clarify the context of the product
                        model.ColumnName = SingleElement.Product + " (" + Sample.SingleElements + ")";
                    }
                    else
                    {
                        model.ColumnName = Sample.ProductName;
                    }

                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Product_Sel(Guid.Parse(model.ValueOld), null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Product_Sel(Guid.Parse(model.ValueNew), null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SampleExamination_id))
                {
                    model.ColumnName = Sample.SampleExamination;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_SampleExamination_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_SampleExamination_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample2CertificationBusiness.CertificationBusiness_id))
                {
                    model.ColumnName = Languages.Pages.Laboratory.Samples.Samples.Sample.CertificationBusiness;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_CertificationBusiness_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_CertificationBusiness_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample2CertificationProduct.CertificationProduct_id))
                {
                    model.ColumnName = Languages.Pages.Laboratory.Samples.Samples.Sample.CertificationProduct;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_CertificationProduct_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_CertificationProduct_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName
                         == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample2CertificationQualitySeal.CertificationQualitySeal_id))
                {
                    model.ColumnName = Sample.CertificationQualitySeal;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_CertificationQualitySeal_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_CertificationQualitySeal_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SampleExaminationCount))
                {
                    model.ColumnName = Sample.SampleExaminationCount;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.LabEntryDate))
                {
                    model.ColumnName = Sample.LabEntryDate;
                    model.ValueOld = string.IsNullOrEmpty(model.ValueOld)
                                         ? string.Empty
                                         : string.Format(Global.DateFormatString, Convert.ToDateTime(model.ValueOld));
                    model.ValueNew = string.IsNullOrEmpty(model.ValueNew)
                                         ? string.Empty
                                         : string.Format(Global.DateFormatString, Convert.ToDateTime(model.ValueNew));
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.CompanySender_id))
                {
                    model.ColumnName = Sample.SenderName;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueOld), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueNew), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.CompanySupplier_id))
                {
                    model.ColumnName = Sample.SupplierName;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueOld), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueNew), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.CompanyManufacturer_id))
                {
                    model.ColumnName = Sample.Manufacturer;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueOld), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueNew), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.CompanyProducer_id))
                {
                    model.ColumnName = Sample.ProducerName;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueOld), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Company_Sel(Guid.Parse(model.ValueNew), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .CompanyName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SamplerUser_id))
                {
                    model.ColumnName = Sample.SamplerUser;
                    var samplerUserOld = model.ValueOld == null
                                             ? new USP_SamplerUser_Sel_Result()
                                             : db.USP_SamplerUser_Sel(Guid.Parse(model.ValueOld), null, null, 0, null, null, currentCulture).Single();
                    model.ValueOld = SamplerUserManagementController.GetSamplerUserTextFieldText(
                        samplerUserOld.FirstName,
                        samplerUserOld.LastName,
                        samplerUserOld.Company_Joined_CompanyName);

                    var sampleUserNew = model.ValueNew == null
                                            ? new USP_SamplerUser_Sel_Result()
                                            : db.USP_SamplerUser_Sel(Guid.Parse(model.ValueNew), null, null, 0, null, null, currentCulture).Single();
                    model.ValueNew = SamplerUserManagementController.GetSamplerUserTextFieldText(
                        sampleUserNew.FirstName,
                        sampleUserNew.LastName,
                        sampleUserNew.Company_Joined_CompanyName);
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SamplingDate))
                {
                    model.ColumnName = Sample.SamplingDate;
                    model.ValueOld = string.IsNullOrEmpty(model.ValueOld)
                                         ? string.Empty
                                         : string.Format(Global.DateFormatString, Convert.ToDateTime(model.ValueOld));
                    model.ValueNew = string.IsNullOrEmpty(model.ValueNew)
                                         ? string.Empty
                                         : string.Format(Global.DateFormatString, Convert.ToDateTime(model.ValueNew));
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.BioProductType_id))
                {
                    model.ColumnName = Sample.BioProductType;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_BioProductType_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_BioProductType_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.BioInstance))
                {
                    model.ColumnName = Sample.BioInstance;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SampleKind_id))
                {
                    model.ColumnName = Sample.SampleKind;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_SampleKind_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_SampleKind_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SampleKindInfo))
                {
                    model.ColumnName = Sample.SampleKindInfo;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SampleType_id))
                {
                    model.ColumnName = Sample.SampleType;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_SampleType_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_SampleType_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SampleStatus_id))
                {
                    model.ColumnName = Sample.Status;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_SampleStatus_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_SampleStatus_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.IsTransferred))
                {
                    model.ColumnName = Sample.IsTransferred;
                    model.ValueOld = bitToFriendly(model.ValueOld);
                    model.ValueNew = bitToFriendly(model.ValueNew);
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.TransferredDate))
                {
                    model.ColumnName = Sample.TransferredDate;
                    model.ValueOld = string.IsNullOrEmpty(model.ValueOld)
                                         ? string.Empty
                                         : string.Format(Global.DateFormatString, Convert.ToDateTime(model.ValueOld));
                    model.ValueNew = string.IsNullOrEmpty(model.ValueNew)
                                         ? string.Empty
                                         : string.Format(Global.DateFormatString, Convert.ToDateTime(model.ValueNew));
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Sample.TransferredBy_Account_id))
                {
                    model.ColumnName = Sample.TransferredAccountName;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Account_Sel(
                                             Guid.Parse(model.ValueOld),
                                             null,
                                             null,
                                             null,
                                             null,
                                             null,
                                             null,
                                             0,
                                             null,
                                             null,
                                             currentCulture).Single().UserName;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Account_Sel(
                                             Guid.Parse(model.ValueNew),
                                             null,
                                             null,
                                             null,
                                             null,
                                             null,
                                             null,
                                             0,
                                             null,
                                             null,
                                             currentCulture).Single().UserName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.FarmingType_id))
                {
                    model.ColumnName = Sample.FarmingType;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_FarmingType_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_FarmingType_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.OfferForm_id))
                {
                    model.ColumnName = Sample.OfferForm;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_OfferForm_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_OfferForm_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.Country_id))
                {
                    model.ColumnName = Sample.Country;
                    if (model.TableName == CustomMetadataProvider.GetMemberName(() => EFHelper.SingleElements))
                    {
                        // Add context if it's SingleElements
                        model.ColumnName += " (" + Sample.SingleElements + ")";
                    }
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Country_Sel(Guid.Parse(model.ValueOld), null, null, 0, null, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Country_Sel(Guid.Parse(model.ValueNew), null, null, 0, null, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SingleElements.Portion))
                {
                    model.ColumnName = SingleElement.Portion;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.CanSingleElementsVary))
                {
                    model.ColumnName = Sample.CanSingleElementsVary;
                    model.ValueOld = bitToFriendly(model.ValueOld);
                    model.ValueNew = bitToFriendly(model.ValueNew);
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.Examined_Product_id))
                {
                    model.ColumnName = Sample.ExaminedIngredient;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Product_Sel(Guid.Parse(model.ValueOld), null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Product_Sel(Guid.Parse(model.ValueNew), null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.BrandMarketingNorm_id))
                {
                    model.ColumnName = Sample.BrandMarketingNorm;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_BrandMarketingNorm_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_BrandMarketingNorm_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SizeType))
                {
                    model.ColumnName = Sample.SizeType;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.AdditionalDetails))
                {
                    model.ColumnName = Sample.AdditionalDetails;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.BrandType_id))
                {
                    model.ColumnName = Sample.BrandType;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_BrandType_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_BrandType_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_PrivateBrand.PrivateBrand_id))
                {
                    model.ColumnName = Sample.Brand;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_PrivateBrand_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_PrivateBrand_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.Brand))
                {
                    model.ColumnName = Sample.Brand;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.EANBox))
                {
                    model.ColumnName = Sample.EANBox;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.EANConsumerPacking))
                {
                    model.ColumnName = Sample.EANConsumerPacking;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.ChargeNumber))
                {
                    model.ColumnName = Sample.ChargeNumber;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.FieldPassNumber))
                {
                    model.ColumnName = Sample.FieldPassNumber;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SampleCount))
                {
                    model.ColumnName = Sample.SampleCount;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SampleQuantity))
                {
                    model.ColumnName = Sample.SampleQuantity;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SingleExamination))
                {
                    model.ColumnName = Sample.SingleExamination;
                    model.ValueOld = bitToFriendly(model.ValueOld);
                    model.ValueNew = bitToFriendly(model.ValueNew);
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.AdditionalExamination))
                {
                    model.ColumnName = Sample.AdditionalExamination;
                    model.ValueOld = bitToFriendly(model.ValueOld);
                    model.ValueNew = bitToFriendly(model.ValueNew);
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.DefaultExamination))
                {
                    model.ColumnName = Sample.DefaultExamination;
                    model.ValueOld = bitToFriendly(model.ValueOld);
                    model.ValueNew = bitToFriendly(model.ValueNew);
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.ReportDate))
                {
                    model.ColumnName = Sample.ReportDate;
                    model.ValueOld = string.IsNullOrEmpty(model.ValueOld)
                                         ? string.Empty
                                         : string.Format(Global.DateFormatString, Convert.ToDateTime(model.ValueOld));
                    model.ValueNew = string.IsNullOrEmpty(model.ValueNew)
                                         ? string.Empty
                                         : string.Format(Global.DateFormatString, Convert.ToDateTime(model.ValueNew));
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.Comment))
                {
                    model.ColumnName = Sample.Comment;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.LotNumber))
                {
                    model.ColumnName = Sample.LotNumber;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PreCalc_CountCertificationBusiness))
                {
                    model.ColumnName = Sample.CountCertificationBusiness;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PreCalc_CountCertificationProduct))
                {
                    model.ColumnName = Sample.CountCertificationProduct;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PreCalc_CountCertificationQualitySeal))
                {
                    model.ColumnName = Sample.CountCertificationQualitySeal;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PreCalc_CountIngredients))
                {
                    model.ColumnName = Sample.CountIngredients;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PreCalc_CountSampleDocuments))
                {
                    model.ColumnName = Sample.CountSampleDocuments;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PreCalc_CountSingleElements))
                {
                    model.ColumnName = Sample.CountSingleElements;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PreCalc_Private_Rating))
                {
                    return null; // ignore this history
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PreCalc_Public_Rating))
                {
                    return null; // ignore this history
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PreCalc_Public_Rating_id))
                {
                    model.ColumnName = Sample.RatingPublic;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Rating_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Rating_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PreCalc_Private_Rating_id))
                {
                    model.ColumnName = Sample.RatingPrivate;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Rating_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Rating_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.PSM))
                {
                    // Pflanzenschutzmittel
                    model.ColumnName = Sample.PSM;
                    model.ValueOld = bitToFriendly(model.ValueOld);
                    model.ValueNew = bitToFriendly(model.ValueNew);
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.ADIValue))
                {
                    model.ColumnName = SampleResult.ADIValue;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.ADIValueUtilisation))
                {
                    model.ColumnName = SampleResult.ADIValueUtilisation;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.ARfDValue))
                {
                    model.ColumnName = SampleResult.ARfDValue;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.ARfDValueUtilisation))
                {
                    model.ColumnName = SampleResult.ARfDValueUtilisation;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.MeasuredValue))
                {
                    model.ColumnName = SampleResult.MeasuredValue;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.RefValue))
                {
                    model.ColumnName = SampleResult.RefValue;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.RefValueUtilisation))
                {
                    model.ColumnName = SampleResult.RefValueUtilisation;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.MeasuredValuePredicate))
                {
                    model.ColumnName = SampleResult.MeasuredValuePredicate;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.EvaluationBasis_id))
                {
                    model.ColumnName = SampleResult.EvaluationBasis;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_EvaluationBasis_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_EvaluationBasis_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.MeasurementPresetting_id))
                {
                    model.ColumnName = SampleResult.MeasuredValueWithMeasurementpresetting;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_MeasurementPresetting_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_MeasurementPresetting_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.Method_id))
                {
                    model.ColumnName = SampleResult.Method;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Method_Sel(Guid.Parse(model.ValueOld), null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Method_Sel(Guid.Parse(model.ValueNew), null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleResult.Parameter_id))
                {
                    model.ColumnName = SampleResult.Parameter;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Parameter_Sel(Guid.Parse(model.ValueOld), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Parameter_Sel(Guid.Parse(model.ValueNew), null, null, null, null, null, 0, null, null, currentCulture)
                                               .Single()
                                               .Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_WeightUnit.WeightUnit_id))
                {
                    model.ColumnName = SampleResult.WeightUnit;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_WeightUnit_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_WeightUnit_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleRating.Rating_id))
                {
                    model.ColumnName = SampleRating.Rating;

                    // Find rating area joined name, since otherwise there is no context
                    var sampleRating = db.USP_SampleRating_Sel(null, model.Value_id, null, null, null, 0, null, null, currentCulture).SingleOrDefault();
                    if (sampleRating != null)
                    {
                        model.ColumnName += " (" + sampleRating.RatingArea_Joined_Name_Trans + ")" + " ("
                                            + sampleRating.RatingArea_TableJoin_RatingSchema_Name_Trans + ")";
                    }

                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_Rating_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture).Single().Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_Rating_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture).Single().Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleRating.RatingArea_id))
                {
                    model.ColumnName = SampleRating.RatingArea;

                    var ratingAreaOld = model.ValueOld == null
                                            ? null
                                            : db.USP_RatingArea_Sel(Guid.Parse(model.ValueOld), null, null, 0, null, null, currentCulture).Single();
                    var ratingAreaNew = model.ValueNew == null
                                            ? null
                                            : db.USP_RatingArea_Sel(Guid.Parse(model.ValueNew), null, null, 0, null, null, currentCulture).Single();

                    model.ValueOld = ratingAreaOld == null
                                         ? string.Empty
                                         : ratingAreaOld.Name_Trans + " (" + ratingAreaOld.RatingSchema_Joined_Name_Trans + ")";
                    model.ValueNew = ratingAreaNew == null
                                         ? string.Empty
                                         : ratingAreaNew.Name_Trans + " (" + ratingAreaNew.RatingSchema_Joined_Name_Trans + ")";
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_SampleDocuments.FileName))
                {
                    model.ColumnName = Document.FileName;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.ProcessingStatus_id))
                {
                    model.ColumnName = Sample.ProcessingStatus;
                    model.ValueOld = model.ValueOld == null
                     ? string.Empty
                     : db.USP_ProcessingStatus_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture)
                           .Single()
                           .Name_Trans;
                    model.ValueNew = model.ValueNew == null
                     ? string.Empty
                     : db.USP_ProcessingStatus_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture)
                        .Single()
                        .Name_Trans;
                }
                else if (model.ColumnName == CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.IntendedUse_id))
                {
                    model.ColumnName = Sample.IntendedUse;
                    model.ValueOld = model.ValueOld == null
                                         ? string.Empty
                                         : db.USP_IntendedUse_Sel(Guid.Parse(model.ValueOld), null, 0, null, currentCulture)
                                             .Single()
                                             .Name_Trans;
                    model.ValueNew = model.ValueNew == null
                                         ? string.Empty
                                         : db.USP_IntendedUse_Sel(Guid.Parse(model.ValueNew), null, 0, null, currentCulture)
                                             .Single()
                                             .Name_Trans;
                }

                // Additional information for SampleResults
                    if (model.TableName == "SampleResults")
                {
                    var result = db.USP_SampleResults_Sel(null, model.Value_id, null, null, 0, null, null, currentCulture).Single();
                    model.ColumnName += " (Parameter: " + result.Parameter_Joined_Name_Trans + ")";
                }

                // catch null or emtpy values
                if (string.IsNullOrEmpty(model.ValueOld) && string.IsNullOrEmpty(model.ValueNew))
                {
                    model.ValueOld = SampleHistory.StringNullOrEmptyHint;
                    model.ValueNew = SampleHistory.StringNullOrEmptyHint;
                }
            }
            catch (InvalidOperationException ex)
            {
                ILog errorLog = LogManager.GetLogger("ErrorLogger");
                errorLog.Error("Parse Sample history fails (InvalidOperationException): " + ex.Message);
            }
            catch (NullReferenceException e)
            {
                // handle null if some db query returns a null object
                ILog errorLog = LogManager.GetLogger("ErrorLogger");
                errorLog.Error("Parse Sample history fails (NullReferenceException): " + e.Message);
            }
            catch (FormatException e)
            {
                // handle format exception if converting a date fails
                ILog errorLog = LogManager.GetLogger("ErrorLogger");
                errorLog.Error("Parse Sample history fails (FormatException): " + e.Message);
            }
            catch (Exception e)
            {
                // handle format exception if converting a date fails
                ILog errorLog = LogManager.GetLogger("ErrorLogger");
                errorLog.Error("Parse Sample history fails (General Exception): " + e.Message);
            }

            return model;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Takes "0" or "1" and returns its friendly name ("yes" or "no"). Otherwise returns the unchanged string.
        /// </summary>
        /// <param name="bit">
        /// </param>
        /// <returns>
        /// </returns>
        private string bitToFriendly(string bit)
        {
            switch (bit)
            {
                case "0":
                    return Global.False;
                case "1":
                    return Global.True;
                default:
                    return bit;
            }
        }
        #endregion
    }
}