﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="GridViewUserSettingsHelper.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Utility
{
    #region usings
    using System;
    using System.Linq;
    using System.Text.RegularExpressions;

    using Fruitmonitoring.Helper.Utility.Interfaces;
    using Fruitmonitoring.Models;

    using log4net;

    #endregion

    /// <summary>
    /// </summary>
    public class GridViewUserSettingsHelper : IGridViewSettingsHelper
    {
        #region Properties
        /// <summary>
        /// </summary>
        public Entities Db
        {
            get
            {
                if (_db == null)
                {
                    _db = new Entities();
                }

                return _db;
            }
        }
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// removes the page index information from the devexpress gridview settings
        /// </summary>
        /// <param name="componentValue">
        /// gridview cookie value
        /// </param>
        /// <returns>
        /// cookie value without page index information
        /// </returns>
        public string RemovePageIndexFromSettings(string componentValue)
        {
            return null;
        }
        #endregion

        #region Fields
        /// <summary>
        /// </summary>
        protected readonly ILog errorLog = LogManager.GetLogger("ErrorLogger");

        // database object
        /// <summary>
        /// </summary>
        private Entities _db;
        #endregion

        #region IGridViewSettingsHelper methods
        /// <summary>
        /// </summary>
        /// <param name="accountGuid">
        /// </param>
        /// <param name="gridView">
        /// </param>
        /// <param name="value">
        /// </param>
        public void SaveUserLayoutToDatabase(Guid accountGuid, string gridView, string value)
        {
            try
            {
                Db.USP_AccountSettings_InsUpd(accountGuid, gridView, value);
            }
            catch (Exception e)
            {
                errorLog.Error("Error in GridViewSettingsHelper " + e.Message);
            }
        }

        /// <summary>
        /// </summary>
        /// <param name="accountGuid">
        /// </param>
        /// <param name="gridView">
        /// </param>
        public void ResetUserLayoutInDatabase(Guid accountGuid, string gridView)
        {
            try
            {
                Db.USP_AccountSettings_Del(accountGuid, gridView);
            }
            catch (Exception e)
            {
                errorLog.Error("Error in GridViewSettingsHelper " + e.Message);
            }
        }

        /// <summary>
        /// </summary>
        /// <param name="accountGuid">
        /// </param>
        /// <param name="gridView">
        /// </param>
        /// <returns>
        /// </returns>
        public string GetUserLayoutForComponent(Guid accountGuid, string gridView)
        {
            try
            {
                var entry = Db.USP_AccountSettings_Sel(accountGuid, gridView).FirstOrDefault();
                if (entry != null)
                {
                    return entry.ComponentValue;
                }
            }
            catch (Exception e)
            {
                errorLog.Error("Error in GridViewSettingsHelper " + e.Message);
            }

            return null;
        }
        #endregion
    }
}