﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="SolutionConfiguration.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Utility
{
    #region usings
    using System;
    using System.Configuration;
    using System.Data.Common;
    using System.Data.SqlClient;
    using System.Text.RegularExpressions;

    using Fruitmonitoring.Models;

    #endregion

    /// <summary>
    /// The solution configuration.
    /// </summary>
    public class SolutionConfiguration
    {
        #region DB enum

        #region Enums
        /// <summary>
        /// The db type.
        /// </summary>
        public enum DB
        {
            LIVE,

            LIVETEST,

            STAGING
        }
        #endregion

        #endregion

        #region Properties
        /// <summary>
        /// Gets the url of the website depending on its connection string.
        /// </summary>
        public static string SITEURL
        {
            get
            {
                if (string.IsNullOrEmpty(siteurl))
                {
                    if (IsServerConnectionString(DB.LIVE))
                    {
                        siteurl = ConfigurationManager.AppSettings["FruitmonitoringLiveUrl"];
                    }
                    else if (IsServerConnectionString(DB.LIVETEST))
                    {
                        siteurl = ConfigurationManager.AppSettings["FruitmonitoringLiveTestUrl"];
                    }
                    else if (IsServerConnectionString(DB.STAGING))
                    {
                        siteurl = ConfigurationManager.AppSettings["FruitmonitoringStagingUrl"];
                    }
                    else
                    {
                        siteurl = ConfigurationManager.AppSettings["FruitmonitoringTestUrl"];
                    }
                }

                return siteurl;
            }
        }
        #endregion

        #region Static Fields
        /// <summary>
        /// Part of the live db name.
        /// </summary>
        private static readonly string LIVEDBNAME = "FruitMonitoringDBOnline";

        /// <summary>
        /// Part of the live test db name.
        /// </summary>
        private static readonly string LIVETESTDBNAME = "FruitMonitoringDBXmlTest";

        /// <summary>
        /// Part of the staging db name.
        /// </summary>
        private static readonly string STAGINGDBNAME = "FruitMonitoringDBStaging";

        /// <summary>
        /// Holds the site url.
        /// </summary>
        private static string siteurl;
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// The is debug.
        /// </summary>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public static bool IsDebug()
        {
            return false;
        }

        /// <summary>
        /// Checks whether or not we're using the connection string that correspond to the given db type.
        /// </summary>
        /// <param name="dbType">
        /// The db type that is checked.
        /// </param>
        /// <returns>
        /// True if the connection string correspond to the given db type.
        /// </returns>
        public static bool IsServerConnectionString(DB dbType)
        {
            return false;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Gets the database name from configuration named "entities".
        /// </summary>
        /// <returns>The database name.</returns>
        private static string GetDatabaseNameFromConfiguration()
        {
            return null;
        }

        /// <summary>
        /// Gets the DB name depend on the DB type.
        /// </summary>
        /// <param name="dbType">The DB type.</param>
        /// <returns>The DB name that corespond to the DB type.</returns>
        private static string GetDbNameFromDbType(DB dbType)
        {
            return null;
        }

        /// <summary>
        /// Should the db name matched exactly?
        /// </summary>
        /// <param name="dbType">DB type to match.</param>
        /// <returns>True if db name should match exactly. False if match should be a "contains".</returns>
        private static bool MatchExact(DB dbType)
        {
            return false;
        }
        #endregion
    }
}