﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="FluentRegistry.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Utility.Tasks
{
    #region usings
    using System;

    using FluentScheduler;

    using Fruitmonitoring.Helper.Modules.EdekaWebService;
    using Fruitmonitoring.Helper.Utility.Tasks.EdekaTransferTask;
    using Fruitmonitoring.Helper.Utility.Tasks.Tasks.SampleOverdueTask;

    using log4net;

    #endregion

    /// <summary>
    /// Registering of ITasks or Actions to be executed at a specific time.
    /// </summary>
    public class FluentRegistry : Registry
    {
        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="FluentRegistry"/> class.
        /// </summary>
        public FluentRegistry()
        {
            _edekaTransferLog = LogManager.GetLogger("EdekaTransferLogger");
            _errorLog = LogManager.GetLogger("ErrorLogger");
            // activate the following tasks only if the application runs on live system
            if (SolutionConfiguration.IsServerConnectionString(SolutionConfiguration.DB.LIVE))
            {
                try
                {
                    runSampleExaminationTasks();
                }
                catch (Exception e)
                {
                    _errorLog.Error("Error registering sample examination task: " + e);
                }
            }

            // activate the edeka task only if the application runs on live system or if the execution is explicitly activated on test system
            var edekaHelper = new EdekaTransferHelper();
            if (SolutionConfiguration.IsServerConnectionString(SolutionConfiguration.DB.LIVE) || edekaHelper.IsEdekaTransferActivatedOnTestSystem())
            {
                try
                {
                    runEdekaSampleTransferTask();
                    _edekaTransferLog.Info(string.Format("Edeka transfer task registered at {0}", DateTime.Now));
                }
                catch (Exception e)
                {
                    _errorLog.Error("Error registering edeka sample tranfser task: " + e);
                }
            }

            // Uncomment for testing - only as long as endpoint is the test adress!
            // Schedule<EdekaSampleTransferTask>().ToRunNow();
            // Schedule<SampleDefaultExaminationTask>().ToRunNow();
            // Schedule<SampleAdditionalExaminationTask>().ToRunNow();
            // Schedule<SampleSingleExaminationTask>().ToRunNow();
            // Schedule<SampleNoExaminationTypeTask>().ToRunNow();
        }
        #endregion

        #region Fields
        /// <summary>
        /// The edeka transfer logger.
        /// </summary>
        private readonly ILog _edekaTransferLog;

        /// <summary>
        /// The error  logger.
        /// </summary>
        private readonly ILog _errorLog;
        #endregion

        #region Methods
        /// <summary>
        /// The run edeka sample transfer task.
        /// </summary>
        private void runEdekaSampleTransferTask()
        {
        }

        /// <summary>
        /// The run sample examination tasks.
        /// </summary>
        private void runSampleExaminationTasks()
        {
        }
        #endregion
    }
}