﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="CompanyEmailRequiredAttribute.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Validation
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.ComponentModel.DataAnnotations;

    using Fruitmonitoring.Helper.Utility;

    #endregion

    /// <summary>
    /// The company email required attribute.
    /// </summary>
    public class CompanyEmailRequiredAttribute : ValidationAttribute
    {
        #region Properties
        /// <summary>
        /// Gets or sets the desired values.
        /// </summary>
        private List<string> DesiredValues { get; set; }

        /// <summary>
        /// Gets or sets the property name.
        /// </summary>
        private string PropertyName { get; set; }

        #region Fields
        /// <summary>
        /// The _inner attribute.
        /// </summary>
        protected RegularExpressionAttribute _innerAttribute;
        #endregion

        #endregion

        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="CompanyEmailRequiredAttribute"/> class.
        /// </summary>
        public CompanyEmailRequiredAttribute()
        {
            _innerAttribute = new RegularExpressionAttribute(ValidationHelper.MASSEMAILREGEX);

            PropertyName = CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Company.CompanyType_JoinedList_CompanyType_id);

            DesiredValues = new List<string>()
                            {
                                DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.RETAILERCOMPANY).ToString(),
                                DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.RETAILERCLIENT).ToString(),
                                DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.SYSOP).ToString(),
                                DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.LABORATORY).ToString(),
                                DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.ADMINISTRATOR).ToString()
                            };
        }
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// Custom error message format.
        /// </summary>
        /// <param name="name">
        /// Fieldname which is not valid.
        /// </param>
        /// <returns>
        /// Formated error message.
        /// </returns>
        public override string FormatErrorMessage(string name)
        {
            return null;
        }
        #endregion

        #region Methods

        #region methods
        /// <summary>
        /// Check if field is required in this context.
        /// </summary>
        /// <param name="value">
        /// Current value.
        /// </param>
        /// <param name="validationContext">
        /// Validation validationContext.
        /// </param>
        /// <returns>
        /// Null if validation succeed, otherwise an Validation Result.
        /// </returns>
        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            return null;
        }
        #endregion

        #endregion
    }
}