﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="ISampleValidationHelper.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Validation.Interfaces
{
    #region usings
    using System;
    using System.Collections.Generic;

    using Fruitmonitoring.Helper.Utility.Interfaces;
    using Fruitmonitoring.Models;

    #endregion

    /// <summary>
    /// The SampleValidationHelper interface.
    /// </summary>
    public interface ISampleValidationHelper
    {
        #region Properties
        /// <summary>
        /// Gets or sets the date helper.
        /// </summary>
        IDateHelper DateHelper { get; set; }
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// The validate attachment.
        /// </summary>
        /// <param name="documents">
        /// The documents.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateAttachment(IEnumerable<USP_SampleDocuments_Sel_Result> documents);

        /// <summary>
        /// The validate bio instance.
        /// </summary>
        /// <param name="bioProductId">
        /// The bioProductId.
        /// </param>
        /// <param name="bioInstance">
        /// The bioInstance.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateBioInstance(Guid bioProductId, string bioInstance);

        /// <summary>
        /// The validate brand.
        /// </summary>
        /// <param name="brandTypeId">
        /// The brand type id.
        /// </param>
        /// <param name="privateBrandSecuredId">
        /// The private brand secured id.
        /// </param>
        /// <param name="brand">
        /// The brand.
        /// </param>
        /// <param name="retailer">
        /// The retailer.
        /// </param>
        /// <returns>
        /// The <see cref="BrandValidation"/>.
        /// </returns>
        BrandValidation ValidateBrand(Guid? brandTypeId, Guid? privateBrandSecuredId, string brand, string[] retailer);

        /// <summary>
        /// The validate certification business.
        /// </summary>
        /// <param name="certificates">
        /// The certificates.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateCertificationBusiness(string[] certificates);

        /// <summary>
        /// The validate certification products.
        /// </summary>
        /// <param name="certificates">
        /// The certificates.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateCertificationProducts(string[] certificates);

        /// <summary>
        /// The validate certification quality seals.
        /// </summary>
        /// <param name="certificates">
        /// The certificates.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateCertificationQualitySeals(string[] certificates);

        /// <summary>
        /// The validate examined product.
        /// </summary>
        /// <param name="examinationCount">
        ///     The examination count.
        /// </param>
        /// <param name="examinedProduct">
        ///     The examined product.
        /// </param>
        /// <param name="singleElements"></param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        ExaminedProductValidation ValidateExaminedProduct(int examinationCount, Guid? examinedProduct, List<USP_SingleElements_Sel_Result> singleElements);

        /// <summary>
        /// Validate Guid None for ExaminedProduct
        /// </summary>
        /// <param name="examinationCount"></param>
        /// <param name="examinedProduct"></param>
        /// <returns></returns>
        ExaminedProductValidation ValidateExaminedProductGuidNone(int examinationCount, Guid? examinedProduct);

        /// <summary>
        /// The validate farming type.
        /// </summary>
        /// <param name="sampleKindId">
        /// The sample kind id.
        /// </param>
        /// <param name="farmingTypeId">
        /// The farming type id.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateFarmingType(Guid sampleKindId, Guid farmingTypeId);

        /// <summary>
        /// The validate investigation order examination.
        /// </summary>
        /// <param name="defaultExamination">
        /// The default examination.
        /// </param>
        /// <param name="additionalExamination">
        /// The additional examination.
        /// </param>
        /// <param name="singleExamination">
        /// The single examination.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateInvestigationOrderExamination(bool defaultExamination, bool additionalExamination, bool singleExamination);

        /// <summary>
        /// Validates, that the private brand belongs to the selected retailer.
        /// </summary>
        /// <param name="privateBrand">The private brand company id.</param>
        /// <param name="retailer">The selected retailes.</param>
        /// <returns>True if private brand belongs to rtailer. Otherwise false.</returns>
        bool ValidatePrivateBrand(USP_PrivateBrand_Sel_Result privateBrand, string[] retailer);

        /// <summary>
        /// The validate region.
        /// </summary>
        /// <param name="companyRegion">
        /// The copmany region.
        /// </param>
        /// <param name="retailerCheckBoxArrayIds">
        /// The retailer check box array ids.
        /// </param>
        /// <param name="retailerHasRegion">
        /// The retailer has region.
        /// </param>
        /// <returns>
        /// The <see cref="RegionValidationStatus"/>.
        /// </returns>
        RegionValidationStatus ValidateRegion(USP_Company_Sel_Result companyRegion, string[] retailerCheckBoxArrayIds, bool retailerHasRegion);

        /// <summary>
        /// The validate report date.
        /// </summary>
        /// <param name="reportDate">
        /// The report date.
        /// </param>
        /// <param name="labEntryDate">
        /// The lab entry date.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateReportDate(DateTime? reportDate, DateTime? labEntryDate);

        /// <summary>
        /// The validate retailer.
        /// </summary>
        /// <param name="retailerCheckBoxArrayIds">
        /// The retailer check box array ids.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateRetailer(string[] retailerCheckBoxArrayIds);

        /// <summary>
        /// The validate retailer contains company.
        /// </summary>
        /// <param name="retailerCheckBoxArrayIds">
        /// The retailer check box array ids.
        /// </param>
        /// <param name="companyGuid">
        /// The company bioProductId.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateRetailerContainsCompany(string[] retailerCheckBoxArrayIds, Guid? companyGuid);

        /// <summary>
        /// The validate retailer selection.
        /// </summary>
        /// <param name="companyTypeIds">
        /// The company type ids.
        /// </param>
        /// <param name="retailerCheckBoxArrayIds">
        /// The retailer check box array ids.
        /// </param>
        /// <param name="companyId">
        /// The company id.
        /// </param>
        /// <param name="companyRefId">
        /// The company ref id.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateRetailerSelection(List<Guid> companyTypeIds, string[] retailerCheckBoxArrayIds, Guid companyId, Guid? companyRefId);

        /// <summary>
        /// The validate sample company factory.
        /// </summary>
        /// <param name="companyFactoryId">
        /// The company factory id.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSampleCompanyFactory(Guid? companyFactoryId);

        /// <summary>
        /// The validate sample company laboratory.
        /// </summary>
        /// <param name="companyLaboratoryId">
        /// The company laboratory id.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSampleCompanyLaboratory(Guid? companyLaboratoryId);

        /// <summary>
        /// The validate sample company producer.
        /// </summary>
        /// <param name="producerId">
        /// The producer id.
        /// </param>
        /// <param name="companyClient">
        /// The company client.
        /// </param>
        /// <returns>
        /// The <see cref="ProducerValidation"/>.
        /// </returns>
        ProducerValidation ValidateSampleCompanyProducer(Guid? producerId, USP_Company_Sel_Result companyClient);

        /// <summary>
        /// The validate sample company sender.
        /// </summary>
        /// <param name="companySenderId">
        /// The company sender id.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSampleCompanySender(Guid? companySenderId);

        /// <summary>
        /// The validate sample company supplier.
        /// </summary>
        /// <param name="companySupplierId">
        /// The company supplier id.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSampleCompanySupplier(Guid? companySupplierId);

        /// <summary>
        /// The validate sample company supplier belongs to retailer.
        /// </summary>
        /// <param name="companySupplierId">
        /// The company supplier id.
        /// </param>
        /// <param name="retailerCheckBoxArrayIds">
        /// The retailer check box array ids.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSampleCompanySupplierBelongsToRetailer(Guid companySupplierId, string[] retailerCheckBoxArrayIds);

        /// <summary>
        /// The validate sample company supplier supplier is client.
        /// </summary>
        /// <param name="companyId">
        /// The company id.
        /// </param>
        /// <param name="companyTypeList">
        /// The company type list.
        /// </param>
        /// <param name="supplierId">
        /// The supplier id.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSampleCompanySupplierSupplierIsClient(Guid companyId, string companyTypeList, Guid? supplierId);

        /// <summary>
        /// The validate sample examination.
        /// </summary>
        /// <param name="sampleExaminationId">
        /// The sample examination id.
        /// </param>
        /// <param name="sampleExaminationCount">
        /// The sample examination count.
        /// </param>
        /// <returns>
        /// The <see cref="SampleExaminationValidationStatus"/>.
        /// </returns>
        SampleExaminationValidationStatus ValidateSampleExamination(Guid sampleExaminationId, int sampleExaminationCount);

        /// <summary>
        /// The validate sample kind.
        /// </summary>
        /// <param name="sampleKindId">
        /// The sample kind id.
        /// </param>
        /// <param name="sampleKindInfo">
        /// The sample kind info.
        /// </param>
        /// <returns>
        /// The <see cref="SampleKindValidationStatus"/>.
        /// </returns>
        SampleKindValidationStatus ValidateSampleKind(Guid sampleKindId, string sampleKindInfo);

        /// <summary>
        /// The validate sample quantity and count not null.
        /// </summary>
        /// <param name="quantity">
        /// The quantity.
        /// </param>
        /// <param name="count">
        /// The count.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSampleQuantityAndCountNotNull(int? quantity, int? count);

        /// <summary>
        /// The validate sample quantity and count range.
        /// </summary>
        /// <param name="value">
        /// The value.
        /// </param>
        /// <param name="minValue">
        /// The min value.
        /// </param>
        /// <param name="maxValue">
        /// The max value.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSampleQuantityAndCountRange(int? value, int minValue, int maxValue);

        /// <summary>
        /// The validate sample sampler user.
        /// </summary>
        /// <param name="samplerUserId">
        /// The sampler user id.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSampleSamplerUser(Guid? samplerUserId);

        /// <summary>
        /// The validate sampling date before lab date.
        /// </summary>
        /// <param name="samplingDate">
        /// The sampling date.
        /// </param>
        /// <param name="labEntryDate">
        /// The lab entry date.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSamplingDateBeforeLabDate(DateTime? samplingDate, DateTime? labEntryDate);

        /// <summary>
        /// The validate sampling date not in future.
        /// </summary>
        /// <param name="samplingDate">
        /// The sampling date.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateSamplingDateNotInFuture(DateTime? samplingDate);

         /// <summary>
        /// The validate processing status.
        /// </summary>
        /// <param name="processingStatus">
        /// The processing status.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateProcessingStatus(Guid processingStatus);

        /// <summary>
        /// The validate intended Use.
        /// </summary>
        /// <param name="intendedUse">
        /// The intended Use.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        bool ValidateIntendedUse(Guid intendedUse);
        #endregion
    }
}