﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="SampleValidationHelper.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Validation
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;

    using DevExpress.XtraPrinting.Native;

    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Helper.Utility.Interfaces;
    using Fruitmonitoring.Helper.Validation.Interfaces;
    using Fruitmonitoring.Infrastructure.AccessRules;
    using Fruitmonitoring.Models;

    #endregion

    /// <summary>
    /// The region validation status.
    /// </summary>
    public enum RegionValidationStatus
    {
        /// <summary>
        /// The valid.
        /// </summary>
        VALID,

        /// <summary>
        /// The invalidwhileregionnull.
        /// </summary>
        INVALIDWHILEREGIONNULL,

        /// <summary>
        /// The invalidwhileregionnotmatchtoselectedretailer.
        /// </summary>
        INVALIDWHILEREGIONNOTMATCHTOSELECTEDRETAILER,

        // INVALIDWHILEREGIONNOTENABLED,
        // INVALIDWHILEMORETHENONERETAILER,
        /// <summary>
        /// The invalidwhilerequired.
        /// </summary>
        INVALIDWHILEREQUIRED
    }

    /// <summary>
    /// The sample kind validation status.
    /// </summary>
    public enum SampleKindValidationStatus
    {
        /// <summary>
        /// The valid.
        /// </summary>
        VALID,

        /// <summary>
        /// The invalidwhilemissingsamplekindinfo.
        /// </summary>
        INVALIDWHILEMISSINGSAMPLEKINDINFO
    }

    /// <summary>
    /// The sample examination validation status.
    /// </summary>
    public enum SampleExaminationValidationStatus
    {
        /// <summary>
        /// The valid.
        /// </summary>
        VALID,

        /// <summary>
        /// The invalid.
        /// </summary>
        INVALID
    }

    /// <summary>
    /// The brand validation.
    /// </summary>
    public enum BrandValidation
    {
        /// <summary>
        /// The validprivatebrand.
        /// </summary>
        VALIDPRIVATEBRAND,

        /// <summary>
        /// The validotherbrand.
        /// </summary>
        VALIDOTHERBRAND,

        /// <summary>
        /// The validnobrand.
        /// </summary>
        VALIDNOBRAND,

        /// <summary>
        /// The invalid.
        /// </summary>
        INVALID,

        /// <summary>
        /// The invalidwhileprivatebrandnotallowed.
        /// </summary>
        INVALIDWHILEPRIVATEBRANDNOTALLOWED,

        /// <summary>
        /// The invalidwhilemissingbrand.
        /// </summary>
        INVALIDWHILEMISSINGBRAND
    }

    /// <summary>
    /// The producer validation.
    /// </summary>
    public enum ProducerValidation
    {
        /// <summary>
        /// The valid.
        /// </summary>
        VALID,

        /// <summary>
        /// The validbutnull.
        /// </summary>
        VALIDBUTNULL,

        /// <summary>
        /// The invalidwhilerequired.
        /// </summary>
        INVALIDWHILEREQUIRED
    }

    public enum ExaminedProductValidation
    {
        VALID,

        INVALID,

        INVALIDORNOSINGLEELEMENTS
    }

    /// <summary>
    /// The sample validation helper.
    /// </summary>
    public class SampleValidationHelper : ISampleValidationHelper
    {
        #region Properties

        #region Fields
        /// <summary>
        /// The date helper.
        /// </summary>
        private IDateHelper dateHelper;
        #endregion

        #endregion

        #region ISampleValidationHelper Members
        /// <summary>
        /// Gets or sets the date helper.
        /// </summary>
        public IDateHelper DateHelper
        {
            get
            {
                if (dateHelper == null)
                {
                    dateHelper = new DateHelper();
                }

                return dateHelper;
            }

            set
            {
                dateHelper = value;
            }
        }
        #endregion

        #region Validate Dates
        /// <summary>
        /// The validate report date.
        /// </summary>
        /// <param name="reportDate">
        /// The report date.
        /// </param>
        /// <param name="labEntryDate">
        /// The lab entry date.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateReportDate(DateTime? reportDate, DateTime? labEntryDate)
        {
            return false;
        }

        /// <summary>
        /// The validate sampling date before lab date.
        /// </summary>
        /// <param name="samplingDate">
        /// The sampling date.
        /// </param>
        /// <param name="labEntryDate">
        /// The lab entry date.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSamplingDateBeforeLabDate(DateTime? samplingDate, DateTime? labEntryDate)
        {
            return false;
        }

        /// <summary>
        /// The validate sampling date not in future.
        /// </summary>
        /// <param name="samplingDate">
        /// The sampling date.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSamplingDateNotInFuture(DateTime? samplingDate)
        {
            return false;
        }
        #endregion Validate Dates

        #region Validate other fields
        /// <summary>
        /// Validate that at least 1 retailer is selected and that the retailers are retailers.
        /// </summary>
        /// <param name="retailerCheckBoxArrayIds">
        /// Current retailer selelction.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateRetailer(string[] retailerCheckBoxArrayIds)
        {
            return false;
        }

        /// <summary>
        /// Check if Retailer contains given companyGuid.
        /// </summary>
        /// <param name="retailerCheckBoxArrayIds">
        /// </param>
        /// <param name="companyGuid">
        /// The company Guid.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateRetailerContainsCompany(string[] retailerCheckBoxArrayIds, Guid? companyGuid)
        {
            return false;
        }

        /// <summary>
        /// Validate that the given company can select the retailers
        /// e.g. retailer can't select other retailers. laboratories can select all retailers
        /// e.g. storage companies can only select the referenced company.
        /// </summary>
        /// <param name="companyTypeIds">
        /// Company type ids of the current loged-in users company.
        /// </param>
        /// <param name="retailerCheckBoxArrayIds">
        /// Current retailer selection id's.
        /// </param>
        /// <param name="companyId">
        /// Current loged-in company id.
        /// </param>
        /// <param name="companyRefId">
        /// The company id referenced by the company.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateRetailerSelection(List<Guid> companyTypeIds, string[] retailerCheckBoxArrayIds, Guid companyId, Guid? companyRefId)
        {
            return false;
        }

        /// <summary>
        /// The validate region.
        /// </summary>
        /// <param name="companyRegion">
        /// The company region.
        /// </param>
        /// <param name="retailerCheckBoxArrayIds">
        /// The retailer check box array ids.
        /// </param>
        /// <param name="retailerHasRegion">
        /// The retailer has region.
        /// </param>
        /// <returns>
        /// The <see cref="RegionValidationStatus"/>.
        /// </returns>
        public RegionValidationStatus ValidateRegion(USP_Company_Sel_Result companyRegion, string[] retailerCheckBoxArrayIds, bool retailerHasRegion)
        {
            return new RegionValidationStatus();
        }

        /// <summary>
        /// Return false if both SampleCount and SampleQuantity are null, return true for all other cases.
        /// </summary>
        /// <param name="quantity">
        /// </param>
        /// <param name="count">
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSampleQuantityAndCountNotNull(int? quantity, int? count)
        {
            return false;
        }

        /// <summary>
        /// Return true if the value is null or within the given range.
        /// </summary>
        /// <param name="value">
        /// </param>
        /// <param name="minValue">
        /// </param>
        /// <param name="maxValue">
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSampleQuantityAndCountRange(int? value, int minValue, int maxValue)
        {
            return false;
        }

        /// <summary>
        /// The validate farming type.
        /// </summary>
        /// <param name="sampleKindId">
        /// The sample kind id.
        /// </param>
        /// <param name="farmingTypeId">
        /// The farming type id.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateFarmingType(Guid sampleKindId, Guid farmingTypeId)
        {
            return false;
        }

        /// <summary>
        /// The validate sample kind.
        /// </summary>
        /// <param name="sampleKindId">
        /// The sample kind id.
        /// </param>
        /// <param name="sampleKindInfo">
        /// The sample kind info.
        /// </param>
        /// <returns>
        /// The <see cref="SampleKindValidationStatus"/>.
        /// </returns>
        public SampleKindValidationStatus ValidateSampleKind(Guid sampleKindId, string sampleKindInfo)
        {
            return new SampleKindValidationStatus();
        }

        /// <summary>
        /// The validate sample examination.
        /// </summary>
        /// <param name="sampleExaminationId">
        /// The sample examination id.
        /// </param>
        /// <param name="sampleExaminationCount">
        /// The sample examination count.
        /// </param>
        /// <returns>
        /// The <see cref="SampleExaminationValidationStatus"/>.
        /// </returns>
        public SampleExaminationValidationStatus ValidateSampleExamination(Guid sampleExaminationId, int sampleExaminationCount)
        {
            return new SampleExaminationValidationStatus();
        }

        /// <summary>
        /// Validate ivnestigation order examination. Returns isValid if at least one the three order types is true, otherwise returns false.
        /// </summary>
        /// <param name="defaultExamination">
        /// DefaultExamination value.
        /// </param>
        /// <param name="additionalExamination">
        /// AdditionalExamination value.
        /// </param>
        /// <param name="singleExamination">
        /// SingleExamination value.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateInvestigationOrderExamination(bool defaultExamination, bool additionalExamination, bool singleExamination)
        {
            return false;
        }

        /// <summary>
        /// The validate sample sampler user.
        /// </summary>
        /// <param name="samplerUserId">
        /// The sampler user id.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSampleSamplerUser(Guid? samplerUserId)
        {
            return false;
        }

        /// <summary>
        /// The validate brand.
        /// </summary>
        /// <param name="brandTypeId">
        /// The brand type id.
        /// </param>
        /// <param name="privateBrandSecuredId">
        /// The private brand secured id.
        /// </param>
        /// <param name="brand">
        /// The brand.
        /// </param>
        /// <param name="retailer">
        /// The retailer.
        /// </param>
        /// <returns>
        /// The <see cref="BrandValidation"/>.
        /// </returns>
        public BrandValidation ValidateBrand(Guid? brandTypeId, Guid? privateBrandSecuredId, string brand, string[] retailer)
        {
            return new BrandValidation();
        }

        /// <summary>
        /// Validates, that the private brand belongs to the selected retailer.
        /// </summary>
        /// <param name="privateBrand">The private brand object.</param>
        /// <param name="retailer">The selected retailes.</param>
        /// <returns>True if private brand belongs to rtailer. Otherwise false.</returns>
        public bool ValidatePrivateBrand(USP_PrivateBrand_Sel_Result privateBrand, string[] retailer)
        {
            return false;
        }

        /// <summary>
        /// The validate bio instance.
        /// </summary>
        /// <param name="bioProductId">
        /// The bio product id.
        /// </param>
        /// <param name="bioInstance">
        /// The bio instance.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateBioInstance(Guid bioProductId, string bioInstance)
        {
            return false;
        }

        /// <summary>
        /// Validate that an examined product is given if single elements are investigated.
        /// Caution: For final validation, this is dependent on ValidateExaminedProductGuidNone.
        /// </summary>
        /// <param name="examinationCount">
        /// Count of examined elements.
        /// </param>
        /// <param name="examinedProduct">
        ///  Id of examined product.
        /// </param>
        /// <param name="singleElements">List of single elements.</param>
        /// <returns>
        /// ExaminedProductValidation.
        /// </returns>
        public ExaminedProductValidation ValidateExaminedProduct(
            int examinationCount,
            Guid? examinedProduct,
            List<USP_SingleElements_Sel_Result> singleElements)
        {
            return new ExaminedProductValidation();
        }

        /// <summary>
        /// Check for default GUID (GuidNone) of ExaminedProduct during final validation.
        /// </summary>
        /// <param name="examinationCount"></param>
        /// <param name="examinedProduct"></param>
        /// <returns></returns>
        public ExaminedProductValidation ValidateExaminedProductGuidNone(int examinationCount, Guid? examinedProduct)
        {
            return new ExaminedProductValidation();
        }

        /// <summary>
        /// Validate that there is at least 1 attachment added for the sample.
        /// </summary>
        /// <param name="documents">
        /// All documents for the sample.
        /// </param>
        /// <returns>
        /// True if count of documents is &gt; 0, else false.
        /// </returns>
        public bool ValidateAttachment(IEnumerable<USP_SampleDocuments_Sel_Result> documents)
        {
            return false;
        }

        /// <summary>
        /// Processing Status cannot be empty or guidnone.
        /// </summary>
        /// <returns>True or False</returns>
        public bool ValidateProcessingStatus(Guid processingStatus)
        {
            return false;
        }

        /// <summary>
        /// Processing Status cannot be empty or guidnone.
        /// </summary>
        /// <returns>True or False</returns>
        public bool ValidateIntendedUse(Guid intendedUse)
        {
            return false;
        }

        #endregion Validate other fields

        #region companyTypeValidation
        /// <summary>
        /// Validate company is not null or "---" (default company).
        /// </summary>
        /// <param name="companyLaboratoryId">
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSampleCompanyLaboratory(Guid? companyLaboratoryId)
        {
            return false;
        }

        /// <summary>
        /// Validate company is not null or "---" (default company).
        /// </summary>
        /// <param name="companyFactoryId">
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSampleCompanyFactory(Guid? companyFactoryId)
        {
            return false;
        }

        /// <summary>
        /// Validate company is not null or "---" (default company).
        /// </summary>
        /// <param name="companySupplierId">
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSampleCompanySupplier(Guid? companySupplierId)
        {
            return false;
        }

        /// <summary>
        /// Check whether a supplier belongs to a retailer.
        /// </summary>
        /// <param name="companySupplierId">
        /// </param>
        /// <param name="retailerCheckBoxArrayIds">
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSampleCompanySupplierBelongsToRetailer(Guid companySupplierId, string[] retailerCheckBoxArrayIds)
        {
            return false;
        }

        /// <summary>
        /// Validate company is not null or "---" (default company).
        /// </summary>
        /// <param name="companySenderId">
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSampleCompanySender(Guid? companySenderId)
        {
            return false;
        }

        /// <summary>
        /// Validate company producer
        /// producer is only required, if the company-client contains the companytype "producer".
        /// </summary>
        /// <param name="producerId">
        /// </param>
        /// <param name="companyClient">
        /// </param>
        /// <returns>
        /// The <see cref="ProducerValidation"/>.
        /// </returns>
        public ProducerValidation ValidateSampleCompanyProducer(Guid? producerId, USP_Company_Sel_Result companyClient)
        {
            return new ProducerValidation();
        }

        /// <summary>
        /// If the client has company type supplier, the IO's supplier has to be the client.
        /// </summary>
        /// <param name="companyId">
        /// Company id of the current loged in user.
        /// </param>
        /// <param name="companyTypeList">
        /// Company type list of the current loged in user.
        /// </param>
        /// <param name="supplierId">
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateSampleCompanySupplierSupplierIsClient(Guid companyId, string companyTypeList, Guid? supplierId)
        {
            return false;
        }
        #endregion companyTypeValidation

        #region certificates
        /// <summary>
        /// The validate certification quality seals.
        /// </summary>
        /// <param name="certificates">
        /// The certificates.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateCertificationQualitySeals(string[] certificates)
        {
            return false;
        }

        /// <summary>
        /// The validate certification products.
        /// </summary>
        /// <param name="certificates">
        /// The certificates.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateCertificationProducts(string[] certificates)
        {
            return false;
        }

        /// <summary>
        /// The validate certification business.
        /// </summary>
        /// <param name="certificates">
        /// The certificates.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool ValidateCertificationBusiness(string[] certificates)
        {
            return false;
        }
        #endregion certificates
    }
}