﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="UniqueSPAttribute.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Validation
{
    #region usings
    using System;
    using System.ComponentModel.DataAnnotations;
    using System.Globalization;
    using System.Linq;

    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Models;

    #endregion

    /// <summary>
    /// Check the uniqueness of an attribute in the database
    /// [UniqueSP("WeightUnit", "Name", Id = "WeightUnit_id", ErrorMessageResourceType = typeof(Languages.GlobalValidation), ErrorMessageResourceName = "Exists")].
    /// </summary>
    public class UniqueSPAttribute : ValidationAttribute
    {
        #region Constructors and Destructors

        #region constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="UniqueSPAttribute"/> class.
        /// constructor.
        /// </summary>
        /// <param name="tableName">
        /// Table name the column.
        /// </param>
        /// <param name="columnName">
        /// Column which holds the value.
        /// </param>
        /// <param name="groupByColumn">
        /// Column name: check uniqueness for a specific group in a column.
        /// </param>
        /// <param name="exists">
        /// Validation return true if the value exists.
        /// </param>
        public UniqueSPAttribute(string tableName, string columnName, string groupByColumn = null, bool exists = false)
            : base(exists ? "\"{0}\" existiert nicht" : "\"{0}\" existiert bereits")
        {
            // default error message
            TableName = tableName;
            ColumnName = columnName;
            GroupByColumn = groupByColumn;
            Exists = exists;
        }
        #endregion

        #endregion

        #region variables
        /// <summary>
        /// Gets the table name.
        /// </summary>
        public string TableName { get; private set; }

        /// <summary>
        /// Gets the column name.
        /// </summary>
        public string ColumnName { get; private set; }

        /// <summary>
        /// Gets the group by column.
        /// </summary>
        public string GroupByColumn { get; private set; }

        /// <summary>
        /// Gets the group by value.
        /// </summary>
        public string GroupByValue { get; private set; }

        /// <summary>
        /// Gets or sets the id.
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// Gets a value indicating whether exists.
        /// </summary>
        public bool Exists { get; private set; }
        #endregion

        #region methods
        /// <summary>
        /// Custom error message format.
        /// </summary>
        /// <param name="name">
        /// Fieldname which is not unique (not valid).
        /// </param>
        /// <returns>
        /// Formatted error message.
        /// </returns>
        public override string FormatErrorMessage(string name)
        {
            return null;
        }

        /// <summary>
        /// Check if an db entry is unique or not.
        /// </summary>
        /// <param name="value">
        /// Current value to check.
        /// </param>
        /// <param name="validationContext">
        /// </param>
        /// <returns>
        /// The <see cref="ValidationResult"/>.
        /// </returns>
        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            return null;
        }
        #endregion
    }
}